%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Template for population PK analysis using SBPOP/Monolix
% Welcome to an implementation of an "industrialized" popPK workflow. 
% 
% The main concept is that of "divide and conquer", applied to the global
% popPK model space. And functionality is provided to just do that.
% 
% The present workflow does not focus on a specific tool but assumes
% the model building can be approached in a scientifically reasonable way,
% with numerically stable estimation tools at the backend.
% 
% This does not mean that we use a generic tool for estimation. The PoC
% implementation is based on Monolix. But NONMEM with SAEM could be easily
% plugged in and also investigated. However, that was out of scope.
%
% Also, it does not mean that all the approaches below are scientifically
% 100% valid and accepted by everyone. I cooked them together based on
% input from many sides and own wishes. It is clear that, with todays
% availability of reasonably numerically stable estimation tools, model
% selection procedures need to be thought through in detail, be tested and
% subsequently implemented and used. We are not fully there yet.
% 
% This implementation, of course, contains some limitations, coming from
% both lack of time on my side, but mostly from currently available
% features in Monolix. Even though all model structures of interest for
% popPK models can be build using MLXTRAN - an ODE based solution for a
% linear model on a huge trial would just not be useful (taking to much
% time). These limitations will dissappear as Monolix evolves. Currently
% they are at version 4.2.1. The next release will already remove most of
% the limitations. And the next-next release will be fantastic (from my
% perspective, implementing this workflow).
% 
% It needs to be mentioned that I do apply the Pareto principle - a fancy
% expression for not wanting to cover 100% of the possible popPK models. I
% am just happy to cover most of the popPK models that appear in day to day
% work in the pharmaceutical industry - and more specifically in my own
% work. 
%
% This template is written as a users guide to enable a new user to follow
% all the steps. So, all green text is documentation.
% 
% Enjoy!
%
% Henning Schmidt
%
% PS: If you have questions, then email me: henning.schmidt@novartis.com
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% DISCLAIMER
% Use this on your own risk. I am not responsible for any harm this script
% (or any other script I might have authored, corrected, commented, edited,
% documented, etc) might do to you or others. If you print it out and eat
% the paper and choke on it - it's not my responsibility. If you get a
% completely wrong model and your boss is unhappy with you - again, it's
% not my responsibility. If my conclusions and suggestions in here are
% stupid or wrong or improvable - well ok, thats's how it is. There is
% active work going on on this and it will be improved over time as things
% settle down and the use of numerically stable algorithms is spread wider,
% community accepts them, and enough experience has been build up on how to
% best make use of them.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Some of the topics covered in this template:
%
%           - Current limitations
%           - Dataset specification
%           - Initialization
%           - Definition of general settings
%           - Graphical data exploration
%           - Some data cleaning
%           - Base model building
%           - Covariance model building
%           - Covariate model building
%           - Assessment of clinical relevance of covariates
%           - Selection of final model
%           - Assessment of fit robustness
% 
% And some more that are not listed here.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Current limitations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Covered:
% -------
% - 1,2,3 compartmental distribution models with linear distribution,
%   linear and linear+saturable elimination 
% - Infusion, bolus and first order absorption into the central
%   compartment can be present in the same dataset.
% - Lag time on first order absorption
% - Additive, proportional, additive+proportional, and exponential residual
%   error models
% - Diagonal and off diagonal correlations between random effects
% - Selected fixed and random effects can be estimated or fixed to user
%   defined values
% - Covariates, both continuous and categorical ones can be considered on
%   any parameter for which random effects are estimated
% - In each model building step you can use default settings for
%   covariates and random effect correlations
%
% Not covered:
% ------------
% - Nonlinear bioavailability will first be implemented when the next
%   release of Monolix (4.2.2) is available - before that it does not make
%   sense
% - Inter-occasion variability is not considered for the moment. Cross-over
%   studies with washout inbetween can still be handled by splitting
%   subjects in different IDs
% - Nonlinear distribution into peripheral compartments is not yet handled.
%   Need to think if this fits the Pareto principle or not ...
% - Other routes of administration than IV and 1st order absorption into
%   central compartment are not covered at the moment. Again, I am waiting
%   for the next releases of Monolix to be able to simplify my code and then
%   I will implement additional routes (0 order absorption and maybe even
%   mixed things).
% 
% Performance limitations:
% ------------------------
% - If both IV and 1st order absorption administration is present in the
%   dataset, an ODE based solution is used, rather than analytic solutions.
%   This even if the underlying structural PK model is linear. This is not
%   nice, but will be changed as soon as Monolix 4.2.3 is out.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Known things to improve
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Allow more general models with improving Monolix versions
%    - 4.2.2: nonlinear bioavailability on 1st order absorption
%             bioavailability on IV administration
%             analytic solution with bioavailability
%    - 4.2.3: analytic solutions if linear model with multiple routes of
%             administration
%
% - Check that Monolix but regarding block diagonal random effect 
%   correlation matrics with not adjacent parameters is fixed
%
% - Improve methodology as more experience is gained using the SAEM
%
% - ...
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Dataset specification
% The workflow requires a certain dataset standard, explained here.
% The dataset specification is generic for popPK and popPKPD. Why having
% different specs for these tasks and overload the programmers with
% uneccessary work?
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% The required columns are the following ones:
%
%   STUD:       Numeric study number   
%   SUBJ:       Numeric subject identifier
%   ID:         Unique subject identifier (to be used for model building)
%   TYPE:       Column with type identifier for each record 
%               Standard assumes dose record has TYPE 0 and PK observation
%               record has TYPE 1
%   TIM2:       Time post first dose (negative values for observation
%               records pre-first dose)
%   TAD:        Time since last dose (pre-first-dose values same as TIM2
%               values)
%   TIMEUNIT:   Column with units of time measurements in dataset
%   DV:         Observation value (0 for dose records, LOQ for values above
%               or below the limit of quantification if CENS not equal 0)
%   UNIT:       Column with units for record entries
%   NAME:       Column with names/labels of record entries
%   MDV:        Missing data value columns (0 if observation value is
%               defined, 1 for dose records and for unknown observation
%               values)
% 	EVID: 		Event ID. 0 for observations, 1 for dosing records.
%   AMT:        Dose given at dosing instant (0 for observation records) 
%   ADM:        Administration column (0 for observation records, 1 for IV
%               infusion or bolus, 2 for first order absorption into
%               central compartment)  
%   DOSE:       This column contains the value of the last dose given. 
%               It is used for dose-normalization of the concentration. 
%   TRT:        Numeric treatment assignment. In case of fixed dose dosing
%               the column contains the same information as the DOSE
%               column. Differences will appear if dosing defined in mg/kg
%               or mg/m2, etc. Then the TRT column will contain the mg/kg
%               or mg/m2 values, while the DOSE column will contain the
%               actual dose levels given to a specific patient.
%               This column is used for stratification of the graphical
%               exploration plots. For placebo subjects TRT=0 should be
%               chosen.
%   REGIMEN:    Numeric regimen assignment. Different regimen should obtain
%               different numeric identifiers. This column is used for
%               stratification of the graphical exploration plots. 
%   CENS:       Censored data (=1 for observation records below lower limit
%               of quantification, =-1 for observation records above upper
%               limit of quantification, =0 for all other records)  
%
% Not required columns, which are used when defined are:
%
%   TINF:       Infusion time. (0 for observation records, 0 for "Bolus" or
%               "first order absorption" dose records, Infusion time in
%               TIMEUNIT unit if infusion dose record).
%
% Additional columns:
%
% - User defined continuous and categorical covariates. Here only time
%   independent covariates should be considered.
%   -----------------------------------------------------------------------
%   This could/should include variables such as disease related biomarkers,
%   since the PK might depend on the PD, concomittant medications, etc.
%   Just mentioning it here because I might forget it otherwise. 
%   -----------------------------------------------------------------------
% - User defined time dependent covariates. These could be used for users
%   own graphical analysis. "Automatic" generation of exploratory plots for
%   these currently not supported.
%
% Columns that might make it into the standard:
% (They are no in yet - but make sense - just putting them here to remind
% myself)
%
% - VISIT:      Visit number
% - BASE:       Baseline flag. =0 for non baseline observation records, 
%               =1 for baseline observation records, =0 for dose records
% - SCREEN:     Screening flag. =0 for non screening observation records, 
%               =1 for screening observation records, =0 for dose records
% - EVDT:       Date/time string for this record.
% - AENAME:     String with adverse event label
% - AEDURATION: Duration of observed adverse event in dataset time unit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Once the dataset is available, start the analysis. In the following we
% start the tutorial by mixing commands to run with documentation what to
% do, how to inteprete results, and how to go to the next step - enjoy the
% ride! 
%
% And if you find things that could be improved, do not hesitate to drop me
% an email: henning.schmidt@novartis.com
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initialization
% Run this section to 
%   - Clean-up completely before starting the analysis.
%   - Install SBPOP from a defined location
%   - Install the MATLAB version of Monolix from a defined location
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Clean all
clc; clear all; close all; restoredefaultpath();

%--------------------------------------------------------------------------
% Please defined the paths where the different softwares can be found on
% your system
%--------------------------------------------------------------------------
path2SBPOP              = '~/TOOLS/SBPOP_Rev_1012';
path2Monolix            = '/CHBS/apps/Monolix/latest/Matlab/matlab';

% Installation of required tools
oldpath = pwd; cd(path2Monolix);    addpath(genpath(pwd));  run_init();     cd(oldpath);
oldpath = pwd; cd(path2SBPOP);      installSBPOPpackage();                  cd(oldpath);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Definition of general settings
% In this section you will specify general settings for your population PK
% analysis. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% Please edit this to define the relative path to the dataset you want to
% use for the analysis. It should be a dataset in the format, explained
% above.
% -------------------------------------------------------------------------
originalDatasetFile                 = '../dataEXAMPLE.csv'; 

% -------------------------------------------------------------------------
% In this part you specify the names of the additional columns in the
% dataset that you want to look at during the graphical exploration and
% which you might subsequently also to consider as candidate covariates.
% 
% "covNames": define column names in the dataset that contain continuous
%             data 
% "catNames": define column names in the dataset that contain categorical
%             data 
% -------------------------------------------------------------------------
covNames                            = {'AGE0'    'HT0'      'WT0'     'BMI0'};
catNames                            = {'SEX'     'RACE'     'ETHN'};

% -------------------------------------------------------------------------
% In the following some data cleaning will be done. It includes automatic
% covariate imputation. For continuous covariates the missing values will
% be imputed to the median of the population in the dataset. For
% categorical covariates default imputation values need to be defined,
% which is done here (the order of the definitions needs to be the same as
% the order of the names in the "catNames" variable above).
catImputationValues                 = [  1         2          1];
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% The original dataset "originalDatasetFile" will be processed in the
% following, leading to the final popPK analysis dataset. Here, you need to
% define the relative path where to store this analysis dataset.
% Normally, you as user do NOT need to change the default setting, which is
% "../Data/popPKK.csv". IF you want to change it you should be knowing what
% you are doing ;-)
% -------------------------------------------------------------------------
analysisDatasetFile                 = '../Data/popPK.csv';

% -------------------------------------------------------------------------
% The last of the general settings to be provided are the Monolix options.
% Not many options can be changed for the moment, but I try to keep things
% simple for the user. 
% IMPORTANT: although is called "options"Monolix, it is good practice to
% define these and not to not do it. Some hints on how to set the options:
% 
% SEED:         The random seed used by Monolix for the estimation runs.
%               Just keep it on this value or choose whatever you want (6
%               digit integer).
% K1:           Number of the first iterations. The default value in
%               Monolix is 500. Further down we will discuss on how to
%               choose a reasonable value for the current modeling task and
%               then change this value if needed. 
% K2:           Number of the second iterations. The default value in
%               Monolix is 200. That often is a reasonable value. Further
%               down we will discuss on how to assess if this value needs
%               change. 
% K1/2_AUTO:    If set to 0 Monolix will always do the defined K1/K2 number
%               of iterations. If set to 1 Monolix will do at maximum the
%               defined K1/K2 iterations but used heuristics to stop before
%               reaching the maximal values. That is a nice feature but for
%               thorough model building: keep these values always on 0.
% NRCHAINS:     Number of (Markov) chains. When the dataset is small, just
%               a few of subjects for instance, replicating the dataset can
%               improve the precision of the estimation. The number of
%               (Markov) chains specifies how many times the dataset need
%               to be replicated. So for us it means, if you have few
%               subjects and/or sparse data: try higher values than 1. 
% LLsetting:    Defines how Monolix will be calculating the log likelihood. 
%               Possible setting: 'linearization', 'importantSampling', or
%               'both'. 
% -------------------------------------------------------------------------
optionsMonolix                          = [];                      
optionsMonolix.algorithm.SEED           = 123456;                  
optionsMonolix.algorithm.K1             = 1000;                    
optionsMonolix.algorithm.K2             = 200;                     
optionsMonolix.algorithm.K1_AUTO        = 0;                       
optionsMonolix.algorithm.K2_AUTO        = 0;                       
optionsMonolix.algorithm.NRCHAINS       = 1;                       
optionsMonolix.LLsetting                = 'importantSampling';         

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Prepare your system for parallel computation
% If "Parallel Computing Toolbox" available on your system can 
% get your desired number of nodes here and in the following estimation
% runs will be done in parallel.
% If "Parallel Computing Toolbox" not available, then remove this whole
% section, since parallel will not be for you - sorry!
%
% Note that the code below is somewhat specific for the Novartis system 
% I use in my day to day work.
%
% For any other system you need to make sure you get the number of nodes 
% you want (ask your sysadmin and Mathworks) ... typically it will involve
% the "matlabpool" command.
%
% The only additional thing you need to do in this section, and which is
% valid on all systems, is to define the path2MonolixStandalone variable 
% with the path to standalone Monolix.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% Novartis system specific part - please fill in the path 
% to your SBPAR folder
% -------------------------------------------------------------------------
path2SBPAR              = '~/TOOLS/SBPAR_Rev_1004';
oldpath = pwd; cd(path2SBPAR);      installSBPAR();                         cd(oldpath);

% -------------------------------------------------------------------------
% Define the number of nodes you desire
% Define the path to the Monolix standalone version (required for parallel 
% computation)
% -------------------------------------------------------------------------
NumberNodes             = 10;
optionsMonolix.PathMonolixStandalone  = '/CHBS/apps/Monolix/4.2.0/Standalone/Monolix.4.2.0-standalone2008b-linux64/bin/Monolix.sh';

% -------------------------------------------------------------------------
% Novartis specific command you would need to adjust to your system
% In there some matlabpool command is called - ask your sysadmin and let 
% mathworks help you - you are paying the license fees ..
% -------------------------------------------------------------------------
SBPOPinitializeParallelComputingMODESIM(NumberNodes);  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load original dataset
% This section load the original dataset that you specified using the
% variable "originalDatasetFile". 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

data = SBPOPloadCSVdataset(originalDatasetFile);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Check original dataset
% Very important step! You need to make sure that the dataset you obtained
% is in the format you have specified. You should do a manual check, by
% opening the dataset in Excel or in any other software of your choice and
% have a look at it. The function "SBPOPcheckDataFormat" is also doing some
% sanity checks, such as:
% - Checking that all required columns are present
% - Checking that certain columns do not have undefined values 
% - Checking that for the same TYPE the UNIT definitions are the same
% - etc.
% Once run, the function either returns an error if not all required
% columns are defined. If all required columns are defined, it assesses
% other things and prints out the results in the MATLAB command window.
% Please have a look at these results and see if there are problems in your
% dataset that might need fixing. 
% AND: do not think this function will pick up ALL potential issues in your
% dataset. Manual checking and thinking is still needed!
% 
% GOAL of this step: Iterate on the programming of your dataset until you
% are happy and confident that the dataset does make sense
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SBPOPcheckDataFormat(data)  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% General graphical exploration of data
% Now where the dataset has been checked and deemed fit for the next
% steps, some graphical exploration of the data is next on the list.
% Since the popPK workflow here uses a standardized dataset specification,
% many plots can be generated more or less automatically.
%
% This, however, does not mean that you can not do own plots - which you
% probably also should do.
% 
% In this section, by default the whole dataset is considered, and the
% following analyses are performed:
% 
% - Summary plots of the individual data on logY and linearY axis
% - Individual plots of the individual data on logY and linearY axis,
%   indicating also dosing times and amounts. Each observation record is
%   annotated by the index in the dataset, by its TAD and DV value.
% - Summary statistics for the chosen covNames and catNames
% - Individual data stratified by TRT/STUD, TRT/REGIMEN, STUD/REGIMEN, both
%   dose normalized and non-dose normalized. And both over TIM2 and TAD.
% - Covariate information. Covariate histograms for the continuous
%   covariates defined in "covNames". And correlation of continuous
%   covariates and categorical with continuous covariates. 
% - Dose normalized plots stratified in different ways, both in logY and
%   linear Y axes, using different colors for different categorical
%   covariate values. Continuous covariates are split in >= median and <
%   median and colored accordingly.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% The following information typically does not need to be edited.
% color:        If set to 1, the plots are done in color. If set to 0, the
%               plots are partly still in color but efforts are done to
%               allow for interpretation also when printed on black and
%               white printers. 
% outputPath:   Relative path from working directory to folder where you
%               would like to store the results of the graphical analysis.
% plotIndividualData: Plots individual profiles both as overview and
%               single. With many subjects that might take some time.
% plotStratifiedData: Plot stratified plots of the data.
% plotCovariateData:  Plot covariate information.
% -------------------------------------------------------------------------
optionsGraphicalExploration                     = [];
optionsGraphicalExploration.color               = 1;
optionsGraphicalExploration.outputPath          = '../Output/DataExploration/';

optionsGraphicalExploration.plotIndividualData  = 1;
optionsGraphicalExploration.plotStratifiedData  = 1;
optionsGraphicalExploration.plotCovariateData   = 1;

SBPOPexplorePopPKdata(data,covNames,catNames,optionsGraphicalExploration)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Interpretation of the general graphical exploration of all data 
% Please have a look at the generated output files.
% Things to consider: 
% - Does the data make sense, based on what you know about the study
%   designs and your dataset specification with respect to all definitions
% - Are there any extreme and obvious outliers?
% - Do some covariates have an effect on the PK profiles? Can there be
%   confounding?
% - Are the covariate values in the correct ranges? Might happen that one
%   study uses cm and another meter to measure the height of a person, or
%   fraction and percentage for hematocrit values, etc.
% - Does the data suggest dose proportionality - or is there a dose effect
%   on the PK? Could it be target mediated drug disposition?
% - Can distribution and terminal elimination be clearly separated? Might
%   suggest that the model to test should have more than one compartment.
% - Do the max and min values of the covariates make sense, given the units
%   they should have been measured in?
% - Are there many subjects which probably had trough PK samples post dose?
% 
% Many more question could be asked when looking at the data - be creative.
%
% Important: After this first analysis you should be able to decide if you
% can be fairly confident that the data in your dataset does make sense. If
% you are not, then go back to data programming and ask for a revision of
% the issues you found. And then restart the process.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Special graphical exploration of data
% In many cases the more or less automatic graphical exploration plots are
% not enough. For example, no automatic separation between PK samples being
% in steady-state and not being in steady-state, are done.
% In this section you can create additional data exploration plots, using
% the same function as above.
%
% Idea: you subset the full data file and rerun the "SBPOPexplorePopPKdata"
%       function on this subset. You can repeat this as often as you want
%       for different subsets of your data. Just specify a different
%       "outputPath" so you do not overwrite the previous plots.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Custom data exploration
% Obviously, such automatic generation of plots can not handle all eventual
% necesities of the user. Real custom plots might be needed, and this is
% the section for them. 
%
% It is good modeling practice to include them here and use the present
% script as the main/parent script for the whole popPK analysis.
%
% Since long plotting code is not nice to have in here and you might even
% want to use R or any other language to do the plots, these additional
% plots should be coded in separate files within the same folder as this
% present script. Please use an understandable naming convention, since at
% some point in time other people might want to look at this popPK project.
% 
% What should be done here, in this section, is to call the additional
% scripts so they can run. These scripts should store their output plots in
% a relevant place, which means in the "../Output/..." folder.
% 
% Running additional MATLAB scripts is straight forward, just calling them
% by their name. R scripts can be executed using the "system" command in
% MATLAB and then calling "Rscript", if installed on your system.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Data Cleaning 
% In the sections above we made sure we understand what is in our dataset
% and that what is in it also does make sense. In this section now we do
% some simple data cleaning. The following things are done (in the
% following order): 
%
% - User selected IDs and records are removed 
% - Placebo subjects are removed from the data (these are identified by
%   TRT=0)
% - Subjects are removed which have not more than a user specified number
%   of PK observations
% - Missing covariates are imputed. For continuous covariates these are
%   imputed to the median of this covariate in the dataset. Further down in
%   model building cvoariates then are centered by the median. Categorical
%   covariates are imputed to the settings defined above in
%   "catImputationValues". This simple approach of covariate imputation can
%   not handle special cases. If the standard imputation here is not for
%   you, please handle that in the "originalDatasetFile" first and then
%   restart the analysis.
% - Observations below LLOQ are removed (if desired) and the removed 
%   records are documented.
% 
% The result is a new dataset "datanew". Additionally, logfiles will be
% written out in the "optionsDataCleaning.outputPath" folder, specified
% below.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% Here you define the subjects you want to have removed from your dataset.
% These are, e.g., identified based on the graphical analysis plots,
% performed above. The syntax for the definition of which ID to remove is
% as shown in this example. For each ID there is one row. The first entry
% needs to be numeric and specifies the ID, the second needs to be a
% string, specifying the reason why this ID is to be removed from the
% dataset. 
% -------------------------------------------------------------------------
removeID  = {
    1   , 'Removed subject 1 as example'
    16  , 'Removed another subject as example'
};

% -------------------------------------------------------------------------
% Here you define the observation records you want to have removed from
% your dataset. These are, e.g., identified based on the graphical analysis
% plots, performed above. The syntax for the definition of which record to
% remove is as shown in this example. For each record to be removed there
% is one row. The first entry needs to be numeric and specifies the index
% of the record in the original dataset, the second needs to be a string -
% specifying the reason why this record is to be removed from the dataset.
% The index of each observation record can easily be obtained from the
% individual graphical exploration plots, where the indexes are shown.
% Alternatively, you can open the datafile in excel and then use the "row
% number minus one" to adjust for the header in the first row.
% -------------------------------------------------------------------------
removeREC = {
     27   , 'Removed record for example purpose only, not due to data inspection'
     126  , 'Removed record for example purpose only, not due to data inspection'
};

% -------------------------------------------------------------------------
% Define minimal number of PK observations per subject for subject to retain
% in the dataset. So if a subject has not more than "Nobs" PK observations,
% it is removed from the dataset.
% -------------------------------------------------------------------------
Nobs = 1;

% -------------------------------------------------------------------------
% The following information typically does not need to be edited.
% outputPath:   Relative path from working directory to folder where you
%               would like to store the results of the cleaning function.
%               It will store some log files there with information about
%               what was removed and why (settings). Also it will report
%               for which IDs which covariates have been imputed on which
%               values.
% removeBLLOQ:  =0: do not remove BLLOQ values, =1: do remove them
% 				BLLOQ: "Below Lower Limit Of Quantification"
% -------------------------------------------------------------------------
optionsDataCleaning                 = [];
optionsDataCleaning.outputPath      = '../Output/DataCleaning/';
optionsDataCleaning.removeBLLOQ     = 1;
%
datanew = SBPOPcleanPopPKdata(data,removeID,removeREC,Nobs,covNames,catNames,catImputationValues,optionsDataCleaning);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Additional Data Cleaning 
% It might make sense to consider some additional data cleaning for several
% reasons. With less numerically stable estimation algorithms, estimations
% could break down or not converge when outliers where present. With newer
% and more numerically stable algorithms, such as the SAEM this is less of
% an issue. 
% Every removal of a data record or subject needs thorough investigation
% and a scientifically sound reason why it was removed. 
% Examples for reasonable removals are: 
% - Data programming added the same record twice
% - Trough PK measurement has a value that is more like Cmax: in this case
%   the trough sample was most likely taken post, rather than pre-dose. Or
%   there could have been an error when merging the PK data with the dose
%   administration record. These things need to be checked.
% Ideally, these data issues already should have been identified above
% during the data exploration and either the original dataset has been
% updated or removals have been done in the last step.
% 
% If you as the user still want to subset the data to remove a certain
% study or regimen - here is the place to do it, if you do not want to
% change the original data file (keeping it as the "master" data file).
% IMPORTANT: "datanew" is the dataset that was produced in the last step.
% If you want to do some additional cleaning, use datanew and assign it
% again to datanew.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Conversion of data to popPK analysis dataset 
% In this section the cleaned data "datanew" is exported to the
% analysisDatasetFile location. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dataheaderMonolix = SBPOPconvert2popPKdataset(datanew,covNames,catNames,analysisDatasetFile);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% MODEL BUILDING
% In the following sections we will consider popPK model building.
% This is done in five main steps:
% 
% - Base model:          The goal with the base model building is to obtain
%                        a structural model that adequately describes the
%                        data. 
% - Covariance model:    The goal with the covariance model building is to
%                        assess correlations between random effects. Very 
%                        important if you want to simulate the model.
% - Covariate model:     The goal with the covariate model building is to
%                        assess the effect of candidate covariates on the
%                        PK parameters. First identifying statistically
%                        significant covariates. 
% - Final model:         The goal is to decide about a final model, by
%                        addressing the question about clinical relevance
%                        of the identified covariates. 
% - Robustness analysis: Here we check if the parameter estimates are
%                        robust with respect to startng from different
%                        initial guesses. It is a step we show here only
%                        for the final model, but it might make sense to
%                        use it after each of the previous steps to
%                        increase confidence in the model selection by
%                        assessing identifiability for different model
%                        candidates.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The considered model parameters in this workflow
%
% The PK parameters that are available for estimation are:
% - F:      Bioavailability for 1st order administration into central
%           compartment 
% - CL:     Linear elimination from central compartment
% - Vc:     Volume of central compartment
% - Q1:     Intercompartmental clearance between central and first
%           peripheral compartment 
% - Vp1:    Volume of first peripheral compartment
% - Q2:     Intercompartmental clearance between central and second
%           peripheral compartment 
% - Vp2:    Volume of second peripheral compartment
% - VMAX:   "VMAX" term of saturable elimination from central compartment
% - KM:     "KM" term of saturable elimination from central compartment
% - ka:     Absorption rate constant for 1st order absorption into central
%           compartment 
% - Tlag:   Lag time for 1st order absorption into central compartment 
%
% Not all parameters will be estimated for all models. A linear 1
% compartment model will, e.g., at maximum consider the parameters
% F,CL,Vc,ka,Tlag. If no 1st order absorption present then only the
% parameters CL and Vc.
%
% Distribution of random effects:
% -------------------------------
% In this workflow it is assumed that all individual PK model parameters
% are log-normally distributed.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initial parameter guesses and other settings
% Before starting with the base model building we need to get an idea about
% reasonable starting guesses for the PK parameters. Also, it would be
% useful to assess what values for the K1 and K2 parameters in Monolix (see
% above) might be reasonable. This is addressed in this section.
%
% Possible scenarios for choosing initial guesses:
% ------------------------------------------------
% 
% - Simplest:   Prior knowledge/model available. Use these values. If no
%               prior knowledge on all parameters, then use reasonable
%               settings. For example if no info on Q2 and Vp2, use the
%               same values as vor Q1 and Vp1. If no information on VMAX
%               and KM, since previous model linear, set these to any
%               values - since you might not be estimating these values
%               anyway. Tlag can be set to a small fraction of the dosing
%               interval. ka could be set according to the dose interval.
%
% - Simulation: If no prior knowledge available and you really can not
%               guess some useful starting conditions a simulation
%               based approach can be used. This approach is detailed
%               below. Note that this approach does work but might be very
%               slow for large datasets. On the other hand, when you have a
%               very large dataset, you probably should have some good
%               prior for the PK parameters from earlier studies and then
%               you can use the "Simplest" approach.
%
% Scenario for choosing reasonable values for K1 and K2:
% ------------------------------------------------------
%
% - Start with setting up the Simulation based approach above and perform a
%   first estimation having a look at the convergence profile. See below.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% Define a base model structure that you think might be adequate for the
% data you just have graphically explored. This is done by filling out the
% information below. In each part you get guidance what it means.
% -------------------------------------------------------------------------

% -------------------------------------------------------------------------
% Initialize the "modeltest" structure - keep this line as it is
% -------------------------------------------------------------------------
modeltest                           = [];                               

% -------------------------------------------------------------------------
% Select the number of compartments (1, 2, or 3) you think you need to
% describe your data - pick "3" if unsure
% -------------------------------------------------------------------------
modeltest.numberCompartments        = 2;                            

% -------------------------------------------------------------------------
% Select a residual error model. Pick 'comb1' if unsure.
% Supported alternatives are: 
% 'const':      additive error model
% 'prop':       proportional error model
% 'comb1':      additive+proportional error model
% 'exp':        exponential error model
% -------------------------------------------------------------------------
modeltest.errorModels               = 'comb1';  
                                                
% -------------------------------------------------------------------------
% Select the type of clearance you want to consider
% 0: linear clearance from central compartment
% 1: linear+saturable (Michaelis Menten type) clearance from central
%    compartment 
% -------------------------------------------------------------------------
modeltest.saturableClearance        = 1;         

% -------------------------------------------------------------------------
% Select if you want to consider lag time on first order absorption. Use
% "0" if unsure 
% 0: do not consider
% 1: do consider
% -------------------------------------------------------------------------
modeltest.lagTime                   = 0;                              

% -------------------------------------------------------------------------
% Define the proportionality factor to convert dose and concentration units
% correctly. If your dose is in mg and you concentration in ng/ml, this
% factor should be set to "1000". If dose in mg and concentration in ug/ml
% then choose "1". For other units you need to do some math to get the
% factor.
% -------------------------------------------------------------------------
modeltest.FACTOR_UNITS              = 1;

% -------------------------------------------------------------------------
% Define initial guesses for the fixed effect parameters
% Just type any values you might think might be in the right order of
% magnitude. Soon we will play around with them anyway.
% The order of the parameters in the "POPvalues0" vector is as defined
% below.
% -------------------------------------------------------------------------
%                                       F,    CL,   Vc,   Q1,   Vp1,  Q2,  Vp2,  VMAX,   KM,   ka_input2, Tlag_input2
modeltest.POPvalues0                = [ 0.5   0.2   2      1     4     1   2     0.5     5     0.2        0.5];

% -------------------------------------------------------------------------
% Define which fixed and which random effects you want to estimate. By
% default select all to be estimated. A "0" means it is not estimated. A
% "1" means it is estimated. Here in this case we select all to be 1.
% The order is as for the initial guesses of the fixed effect parameters
% above.
% -------------------------------------------------------------------------
modeltest.POPestimate               = [  1     1    1      1     1     1   1      1      1      1           1];
modeltest.IIVestimate               = [  1     1    1      1     1     1   1      1      1      1           1];

% -------------------------------------------------------------------------
% Set some option, to force the function "SBPOPbuildPopPKModelSpace" to
% only create the project but not to run the estimation and produce GOF
% results.
% -------------------------------------------------------------------------
optionsModelSpace                   = [];
optionsModelSpace.buildModelsOnly   = 1;

% -------------------------------------------------------------------------
% Lets create the defined model project - no need for the user to
% change anything here 
% -------------------------------------------------------------------------
SBPOPbuildPopPKModelSpace('MODEL_0_TEST', modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix, optionsModelSpace);

% -------------------------------------------------------------------------
% The previous command generated a Monolix project in the folder 
% "../Models/MODEL_0_TESTmodel" with the name "FITMODEL_0_TEST_001"
%
% The next steps will be manual and will be the following
% - Start the Monolix GUI
% - Load the "FITMODEL_0_TEST_001" project
% - Click on the button "Check initial fixed effects"
% - Play around to see the influence of changes in certain parameters until
%   you get some idea about reasonable initial guesses. This "tutorial"
%   template is not about Monolix, so I assume you know what I want you to
%   do.
% - Once you got some idea about reasonable starting guesses, note them
%   down below and use them in the following steps.
% - In order to assess reasonable values for K1 and K2, keep the Monolix
%   GUI open and run some estimation using the previously identified
%   reasonable initial guesses for the fixed effects. Have a look at the
%   convergence plot and make up your mind if K1=500 is enough or if you
%   better go for larger values. Do the same for K2 but experience so far
%   has shown that K2 is less important. But try it out.
% - Once you have the estimation results - you might as well want to use
%   these as initial guesses.
% 
% Please note that the "Check initial fixed effects" assessment can be VERY
% slow if the dataset is large. But, as mentioned above - then you should
% already have some reasonable guess for the parameters based on previous
% analyses on earlier studies - use these then and only run the estimation
% to get an idea about good settings for K1 and K2. 
% 
% You might even want to consider to use some non-compartmental approaches
% to get ideas about parameter values for central compartment volume,
% clearance, etc.
% -------------------------------------------------------------------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Base model building
% In this section we are going to build our base model. The function
% ("SBPOPbuildPopPKModelSpace") to generate the model candidates is the
% same as we used in the previous section to assess the initial guesses, K1
% and K2. Here we will use it to span a sub model-space to investigate.
%
% It is now important that you select the model-space properties, based on
% the results of the graphical analysis and potential prior knowledge.
% 
% After the generation of the model space, the function
% "SBPOPbuildPopPKModelSpace" will run all the model projects, prepare
% logfiles that will allow to compare the different models based on several
% metrics. And it will produce several graphics with individual fits,
% goodness of fit assessments, etc. These are discussed later. Now we focus
% on spanning the "Base model space".
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% Initialize the "modeltest" structure - keep this line as it is
% -------------------------------------------------------------------------
modeltest                           = [];                             

% -------------------------------------------------------------------------
% Select the number of compartments you want to test. Use a vector to
% define several. 
% -------------------------------------------------------------------------
modeltest.numberCompartments        = [2 3];                          

% -------------------------------------------------------------------------
% Select residual error models to test. 
% Supported alternatives are: 
% 'const':      additive error model
% 'prop':       proportional error model
% 'comb1':      additive+proportional error model
% 'exp':        exponential error model
% -------------------------------------------------------------------------
modeltest.errorModels               = {'const','prop','comb1'};        

% -------------------------------------------------------------------------
% Select the types of clearance you want to consider
% 0: linear clearance from central compartment
% 1: linear+saturable (Michaelis Menten type) clearance from central
%    compartment 
% -------------------------------------------------------------------------
modeltest.saturableClearance        = [0 1];                   

% -------------------------------------------------------------------------
% Select if lag time on first order absorption should be considered.
% "0" if unsure 
% 0: do not consider
% 1: do consider
% -------------------------------------------------------------------------
modeltest.lagTime                   = [0 1]; 

% -------------------------------------------------------------------------
% We know that the dose is given in mg and the concentration is measured
% in ug/ml. => FACTOR_UNITS=1 
% If dose in mg and concentration in ng/ml you would need to set it to
% 1000.
% -------------------------------------------------------------------------
modeltest.FACTOR_UNITS              = 1;

% -------------------------------------------------------------------------
% We determined reasonable initial guesses for the fixed effect parameters
% above. Lets use them here.
% -------------------------------------------------------------------------
% Define initial guesses for the fixed effect parameters
%                                       F,    CL,   Vc,   Q1,   Vp1,  Q2,  Vp2,  VMAX,   KM,   ka_input2, Tlag_input2
modeltest.POPvalues0                = [ 0.41    0.4     3       0.93    4.66    0.93    4.66    4.4     1.7     0.25    0.5 ];

% -------------------------------------------------------------------------
% In the base model building case, do always allow for estimation of all
% fixed and all random effect parameters. We will narrow down this in the
% following steps if needed.
% -------------------------------------------------------------------------
modeltest.POPestimate               = [ 1       1       1       1       1       1       1       1       1       1       1   ];
modeltest.IIVestimate               = [ 1       1       1       1       1       1       1       1       1       1       1   ];

% -------------------------------------------------------------------------
% Set initial guesses for the random effect - if not done they start at 1
% -------------------------------------------------------------------------
modeltest.IIVvalues0               = [0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5  ];

% -------------------------------------------------------------------------
% Lets run the base model building step.
% It will take a while. You can follow the progress on the screen. The 
% convergence plot is an alternative way of selecting values for K1 and K2. 
% If you run the analysis in parallel mode, it will be faster but no 
% convergence plot will be shown.
% -------------------------------------------------------------------------
SBPOPbuildPopPKModelSpace('MODEL_1_BASE', modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Information about created model projects and output:
% 
% Generated models:
% -----------------
% Please have a look in the "../Models" folder. You will see that the
% function call above has created the folder "MODEL_1_BASEmodel" folder,
% which explains the use of the "MODEL_1_BASE" argument in the above
% function call.
% Go into that folder. You will see a file called "modelInfo.txt". This
% contains a table, specifying which models have been generated
% automatically with all the settings made above.
% The folders in "../Models/MODEL_1_BASEmodel/" each correspond to one
% Monolix project with the settings as defined in the "modelInfo.txt" file.
%
% Monolix results:
% ----------------
% When Monolix is finished with a model run, the results will be
% written into the "../Models/MODEL_1_BASEmodel/<project>/RESULTS" folder.
%
% Generated fit analysis results:
% -------------------------------
% When the "SBPOPbuildPopPKModelSpace" is finished, please have a look in
% the folder "../Output/FitAnalysis". You will see that in there the folder
% "MODEL_1_BASEmodel" has been generated. Change into this folder. You will
% see too subfolders "Info" and "Plots".
% Contents of the two folders:
% Info:     It contains 3 text files. In the case of the base model
%           building step only the file "fitInfoParameters.txt" needs to be
%           considered. If covariances and/or covariates already have been
%           taken into account during base model building then also the two
%           other are of interest. For tutorial purposes we will here only
%           have a look at "fitInfoParameters.txt" and consider the other
%           files further down.
% Plots:    This folder contains fit analysis plots, etc. for each of
%           the projects that have been run. Please have a look at them to
%           see their contents. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Interpretations of the results of the base model building step
% The main goal of the base model step is to find structural models that
% describe the data adequately well. We need to look at the results and
% decide which models to carry forward in the next step of model building.
% Additionally, different error models are assessed. 
%
% The following assessment of results is based on some heuristics and up
% for discussion. 
%
% Step 1: Open the "fitInfoParameters.txt" file. It contains 5 tables. The
%         first is a summary of several different metrics for the different
%         models, ordered by the BIC (lowest on top). The following ones
%         are the estimates for the fixed effects, followed by their
%         relative standard errors, followed by the estimates for the
%         random effects, followed by their relative standard errors.
%         - Have a look at the BIC and make up your mind which models to
%           not consider further. Don't be a slave to the objective
%           function value. Only remove models from consideration if the
%           BIC CLEARLY is much higher.
%         - Have a look at NaN_RSE if it is 1 it means that there are NaN
%           values somewhere in the relative standard errors. 
%         - Have a look at maxRSE(fixedE), which is the maximum relative
%           standard error for the fixed effect estimates, and remove
%           models from consideration which have a RSE above a certain
%           limit (e.g. 40%-50% but that is project specific)
%         - Have a look at maxRSE(randE), which is the maximum relative
%           standard error for the random effect estimates, and remove
%           models from consideration which have a RSE above a certain
%           limit (e.g. 80% but that is project specific)
%         - Have a look at the maximum random effect max(randE) - it might
%           guide you further. 
% Step 2: Have a look at the retained models after Step 1. Do they share
%         common features? Number of compartments, type of error model,
%         type of clearance and lag time? Most likely they do.
% Step 3: Have a look at the retained models after Step 1. Are the
%         parameter estimates (fixed, random) and their relative standard
%         errors somewhat in the same range? If not then that might hint at
%         an identifiability issue or a model misspecification.
% Step 4: Have a look at the goodness of fit plots for the retained models.
%         Do they look reasonable? Hopefully! If not, that can have
%         different reasons: 1) stuck in a local minimum - you would need
%         to refine your initial guesses, 2) error in the dataset - maybe
%         some subjects get subcut but it is flagged as IV, etc. -
%         investigate, 3) model structure not adequate to capture the data
%         signature. Try a different model structure. If not feasible using
%         these workflow tools, then step out of the workflow and try
%         additional model structures manually. 
%         Have a look at the random effects information and look at the
%         shrinkage. If its very high, you might want to consider not
%         estimating the corresponding random effects.
% Step 5: If you found a couple of models that describe the data well, do
%         not just select one but several to continue. For example if both
%         2 and 3 compartments give reasonable results - then continue with
%         2 and 3 compartments. If you do not yet want to select between
%         using lag time or not - take both to the next stage. So the
%         approach is not to select the "best base model" but a "good
%         model sub space" to consider in the next model building
%         step. The main reason for doing this is that the "best base
%         model" might contain more parameters than the next best. Adding
%         in subsequent model building steps more parameters might destroy
%         the identifiability of the "best base model" and you would need
%         to restart. 
%
% Also look at the results.ps file in the RESULTS folder of your selected 
% models. It shows the convergence plot of the SAEM algorithm. Another way
% to check if K1 and K2 are chosen appropriately.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Covariance model building
% In this section the goal is to assess correlations of random effects.
% This is an important need if the model at the end is planned to be used
% for simulation.
% In the following, again the function "SBPOPbuildPopPKModelSpace" will be
% used, so documentation is only given to the parts which are new.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modeltest                           = [];                   
modeltest.numberCompartments        = [2 3];                
modeltest.errorModels               = {'comb1'};            
modeltest.saturableClearance        = [1];                         
modeltest.lagTime                   = [0];                         
modeltest.FACTOR_UNITS              = 1;

% -------------------------------------------------------------------------
% We use the same initial guesses as for base model building here. If you
% want you can update the initial guesses based on the base model results.
% The nice thing about a numerically stable estimation algorithm is of
% course that initial guesses only need to be in the "right ballpark"
% -------------------------------------------------------------------------
%                                       F,    CL,   Vc,   Q1,   Vp1,  Q2,  Vp2,  VMAX,   KM,   ka_input2, Tlag_input2
modeltest.POPvalues0                = [ 0.41    0.4     3       0.93    4.66    0.93    4.66    4.4     1.7     0.25    0.5 ];

% -------------------------------------------------------------------------
% For the base model building we considered the estimation of all fixed
% effects. In this example we will continue to do so also for the
% covariance modeling step. However, you have the option to change that.
% Maybe you saw that a fixed effect was badly estimated and you see from
% the data that not much information might be present. Then you can fix
% this parameter to the initial guess by selecting not to estimate it (a
% "0" value in the following vector):
% -------------------------------------------------------------------------
modeltest.POPestimate               = [ 1       1       1       1       1       1       1       1       1       1       1   ];

% -------------------------------------------------------------------------
% In the base model we selected to estimate random effects on all
% parameters - we could have done differently, but in this example we did.
% Ultimately it is up to you what to do. If based on the data analysis you
% know that very little information is in the data about a certain
% parameter, then you might choose accordingly.
% Here we show that it is possible to consider different scenarios of
% random effect estimation.
% -------------------------------------------------------------------------
modeltest.IIVestimate               = {[1       1       1       1       1       1       1       1       1       1       1   ]
                                       [0       1       1       1       1       1       1       1       1       0       0   ]
                                       [0       1       1       1       1       1       1       1       0       0       0   ]};

% -------------------------------------------------------------------------
% Initial guesses can only be defined globally ...
% -------------------------------------------------------------------------
modeltest.IIVvalues0               = [0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5  ];
                                   
% -------------------------------------------------------------------------
% Here we define the covariances/correlations to test. The user needs to
% specify manually what to test. Covariances are only included for
% parameters that actually are in the tested model and random effects are
% estimated for these. But that is handled automatically. 
% The definitions below should be understood as "covariance setting X is
% tested if the parameters in this setting are actually in the model".
% 
% Each row in this {} matrix is a string and defines the structure of the
% covariance matrix to test. Examples:
% 'diagonal' is a reserved setting for diagonal matrix only. 
% '{CL,Vc}' will only test correlation between CL and Vc.
% '{CL,Vc},{VMAX,KM}' defines a block diagonal covariance matrix
%
% Define apriori what you want to test and then just write down everything
% in this part. You might want to consider the Random Effects plot in the
% fit analysis from the base model to get a starting guess.
%
% Some more experience with the SAEM algorithm still needs to be gained.
% But as rule of thumb, based on few examples:
% - Start with a full covariance matrix (all parameters for which random
%   effects are estimated)
% - If you get error messages like "Matrix is not positive definite" or 
%   "Some parameters have invalid initial values for the given transform"
%   then do not consider correlations between random effects for 
%   which little information is in the data (high shrinkage).
% - You can already specify from the start that you are not going to test
%   correlations between random effects where at least one has a high 
%   shrinkage.
% - Also, estimation of larger covariance matrices requires a lot of
%   data to be available. So if there are not very many subjects in the
%   study you might want to additionally test smaller blocks in a
%   blockdiagonal matrix. {CL,Vc}, e.g., would make sense to also test. But
%   be creative, ...
% -------------------------------------------------------------------------
modeltest.covarianceModels          = {'diagonal', ...
                                       '{CL,Vc}', ... 
                                       '{CL,Vc,Q1,Vp1}', ... 
                                       '{CL,Vc},{VMAX,KM}', ... 
                                       '{CL,Vc,Q1,Vp1},{VMAX,KM}', ... 
                                       };

% -------------------------------------------------------------------------
% Lets run the covariance model building step.
% It will take a while. You can follow the progress on the screen. The 
% convergence plot is an alternative way of selecting values for K1 and K2. 
% If you run the analysis in parallel mode, it will be faster but no 
% convergence plot will be shown.
% -------------------------------------------------------------------------
SBPOPbuildPopPKModelSpace('MODEL_2_COVARIANCE', modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Information about created model projects and output:
% 
% Generated models:
% -----------------
% Please have a look in the "../Models" folder. You will see that the
% function call above has created the folder "MODEL_2_COVARIANCE" folder,
% which explains the use of the 'MODEL_2_COVARIANCE" argument in the above
% function call.
%
% Monolix results:
% ----------------
% When Monolix is finished with a model run, the results will be
% written into the "../Models/MODEL_2_COVARIANCE/<project>/RESULTS" folder.
%
% Generated fit analysis results:
% -------------------------------
% When the "SBPOPbuildPopPKModelSpace" is finished, please have a look in
% the folder "../Output/FitAnalysis". You will see that in there the folder
% "MODEL_2_COVARIANCE" has been generated. Change into this folder. You
% will see two subfolders "Info" and "Plots".
%
% Rest is similar to the explanations for the base model building results.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Interpretations of the results of the covariance model building step
% The main goal of the covariance model step is to assess correlations of
% random effects. This is an important need if the model at the end is
% planned to be used for simulation.
% We need to look at the results and decide which models to carry forward
% in the next step of model building. 
%
% The following assessment of results is based on some heuristics and up
% for discussion. 
%
% Step 1: Open the "fitInfoParameters.txt" file. Assess its contents as
%         described for the base model building step.
%         Additionally you should consider the columns "maxRSE(corr)" and 
%         "max(|corr|)".
%         "maxRSE(corr)": shows the maximum relative standard error for
%         estimates of the correlation between random effects. It will tell
%         you if enough information is present. Very large "max(|corr|)"
%         values here are often due to very small correlations between
%         random effects. In the case that you assess larger blocks of
%         covariances this still might be ok and needed to retain such
%         fits. Afterwards you would conclude that single off-diagonal
%         elements are 0. 
%         "max(|corr|)": shows the maximum absolute value of the estimated
%         correlations. If it is 1 or very close to 1 you should be
%         careful. Seems like choosing a larger block for the covariance
% 		  is more robust under SAEM - more experience with SAEM required.
% Step 2: Have a look at the "fitInfoCovariances.txt" file. It contains the
%         estimated correlation matrices for all the tested models. Check
%         that estimates for correlations are somehow reasonably close
%         between different models in which same off-diagonal elements have
%         been tested. If very different, then this might be a sign of
%         non-identifiability. Also, check against the fit analysis plots
%         from the base model building - are the estimated individual
%         parameters (in terms of the "ETAs") somewhat similarly
%         correlated? Do this comparison for the models you retained after
%         Step 1.
% Step 3: Have a look at the goodness of fit plots for the retained models.
% Step 4: As during base model building, select a "good model sub space" to
%         continue with the covariate model building.
% Step 5: Maybe perform a robustness assessment for a chosen covariance
% 		  setting and have a look how different the estimated correlations
% 		  are. How to do a robustness assessment is described towards the
% 		  end of this script.
%
% Also look at the results.ps file in the RESULTS folder of your selected 
% models. It shows the convergence plot of the SAEM algorithm. Another way
% to check if K1 and K2 are chosen appropriately.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Covariate model building
% In this section we will assess covariates. This will be done in several
% steps.
%
% Step 1: Discuss with the team which candidate covariates are of
%         clinical interest. Select these and specify them in your modeling
%         plan - which candidate covariates and on which parameters each of 
%         these covariates should be tested. 
% Step 2: Consider the correlations between the retained candidate
%         covariates. This information is generated automatically in the
%         data exploration part above. Remove candidate covariates / parameter 
%         pairs until no strong correlation between the tested covariates on
%  		  a certain parameters exists anymore.
% 	      Dont be to strict - consider the compounds pharmacology and the 
% 		  population of subjects in your dataset to make the right choices. 
% 		  Might be that VMAX should depend on gender, so weight will most 
% 		  likely be a statistically significant covariate - but using it 
% 		  instead of gender on VMAX is not the way to go. 
%         Confirm with the team that your selection does still make sense.
% Step 3: Perform a single estimation run where you put all candidate 
% 		  covariates of interest on all parameters. The selection of 
% 	      candidate covariates might be parameter dependend as discussed
% 		  above. 
% Step 4: Assess if the estimated covariate coefficients are different from
%         zero (95% confidence interval does not include 0). These are the
%         statistically significant covariates in your model. 
% Step 5: Retain only the statistically significant candidate covariate / 
%         parameter pairings in your model and re-run the estimation to 
% 		  obtain refined covariate coefficients.
% Step 6: If you want to test correlated candidate covariates on the same 
%         parameter (e.g. weight and gender) or you want to do a sensitivity 
% 		  analysis (e.g. ethnic sensitivity), then create a new modelspace, 
%         fixing the covariate 
%         coefficients of the previoulsy identified statistically 
%         significant covariates and assess only the additional effect of 
%         the previoulsy not considered correlated candidate covariates.
% Step 7: Assess clinical relevance of the identified statisically
%         significant covariates. This can be done in different ways. Here
%         2 approaches are outlined. 1) If a covariate changes the value of
%         a parameter by more than 20% from its population estimate when
%         considering 90% of the population in the dataset, then it is
%         assumed to be clinically relevant. Since this is rather an
%         arbitrary definition, a better approach is 2) Perform simulations
%         of the model to assess how much impact a certain covariate
%         actually has.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% At this stage steps 1-2 should have been completed. The next steps will
% be 4 - 6.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% Define the models to test, as documented above
% -------------------------------------------------------------------------
modeltest                           = [];                       

modeltest.numberCompartments        = [2];                      
modeltest.errorModels               = {'comb1'};                
modeltest.saturableClearance        = [1];                      
modeltest.lagTime                   = [0];                      

modeltest.FACTOR_UNITS              = 1;

%                                       F,    CL,   Vc,   Q1,   Vp1,  Q2,  Vp2,  VMAX,   KM,   ka_input2, Tlag_input2
modeltest.POPvalues0                = [ 0.41    0.4     3       0.93    4.66    0.93    4.66    4.4     1.7     0.25    0.5 ];
modeltest.POPestimate               = [ 1       1       1       1       1       1       1       1       1       1       1   ];
modeltest.IIVestimate               = {[0       1       1       1       1       1       1       1       0       0       0   ]};
modeltest.IIVvalues0                = [0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5  ];

modeltest.covarianceModels          = {'diagonal', ...
                                       '{CL,Vc}'
                                       };

% -------------------------------------------------------------------------
% Additionally, define which covariates to test on which parameters
% Several different combinations can be provided and will be tested on the
% model subspace, defined with the settings above.
% This allows for "manual" covariate search, but also for "full covariate
% search". 
%
% The syntax is as follows. Each row below corresponds to one setting of
% covariates to test. Each element in each row, separated by "{}" defines a
% PK model parameter and the covariates to add to it. First element is the
% PK parameter, following, comma separated elements, are the covariates to
% add to this parameter. Keeping it empty will not test covariates.
% -------------------------------------------------------------------------
modeltest.covariateModels           = {'',...
                                       '{CL,AGE0,WT0}, {Vc,AGE0,WT0}, {VMAX,AGE0,WT0}', ...
                                       };

% -------------------------------------------------------------------------
% Lets run the covariate model building step.
% -------------------------------------------------------------------------
SBPOPbuildPopPKModelSpace('MODEL_3_COVARIATE', modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix);

% -------------------------------------------------------------------------
% In case you want to set initial guesses of covariate coefficients to 
% certain values, use the modeltest.covariateModelValues setting.
%
% In case you want to fix covariate coefficients to certain values, use 
% modeltest.COVestimate.
% 
% Below is a commented example:
% modeltest.covariateModels          = {'{CL,AGE0,WT0}, {Vc,AGE0,WT0}, {VMAX,AGE0,WT0,SEX}'};
% modeltest.covariateModelValues     = {     [0    0]       [0    0]          [0  1.2  0]};
% modeltest.COVestimate              = {     [1    1]       [1    1]          [1  0    1]};
%
% This will additionally test SEX on VMAX but keeping the covariate 
% coefficient of WT0 on VMAX fixed on a value of 1.2.
% -------------------------------------------------------------------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Interpretations of the results of the covariate model building step
% The interpretations of the model fits generated for the covariate search
% is pretty much the same as for the other model building steps above.
% The main difference is that you should also consider the summary
% information provided to you in the "fitInfoCovariates.txt" file.
% Essentially it contains for each of the generated models the estimated
% correlation coefficients with their 95% confidence intervals. An "X" is
% printed behind each of these intervals which does not contain "0".
%
% Covariates that are significant on a parameter for one model and not
% significant on the same parameter on a different model might indicate
% identifiability issues. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Assessing clinical relevance of identified covariates
% Statistical significance doesnt mean these covariates really do make
% sense in the final model. We will need to test their "clinical
% significance". This should not be done by the modeler alone. Discussions
% with the team are necessary to define what in this case "clinically
% significant" means. Once you know that you can do some analyses on your
% most preferred models from the steps before.
% Since it is very hard to in advance generalize this process, we will
% simply discuss 2 simple assessments in the following. They might be
% useful to you or you might want to disregard them completely - up to you.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Covariate effects on parameters
% Here we will test how much the PK parameters change due to the estimated
% covariate coefficients.
% For continuous covariates the range from 5% to 95% quantile of the
% covariates in the dataset are considered.
% For categorical covariates all distinct values are used.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% Select a specific model fit to run this analysis for.
% This is done by providing the path to the Monolix project folder of the
% model.
% -------------------------------------------------------------------------
FIT_PATH = '../Models/MODEL_3_COVARIATEmodel/FITMODEL_3_COVARIATE_004';

% -------------------------------------------------------------------------
% Do the analysis
% -------------------------------------------------------------------------
SBPOPcovariateChangeAssessment(FIT_PATH, analysisDatasetFile)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Interpretation of the results
% Basically up to you. Clearly, the larger the changes that are introduced
% by the covariates, the more important it might be to consider the effect.
% However, try to come up with a more useful metric to assess clinical
% relevance - and do a simulation approach.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Simulation approach to investigate clinical relevance of covariates
% No standard functionality is implemented yet for that in this workflow
% toolbox. However, SBPOP provides a lot of functionality to code that
% easily. But that is out of scope for this popPK workflow tutorial.
%
% Alternative: 
% ============
% Open Monolix, load the model project you are interested in and generate
% the diagnostic VPC. It allows you to stratify by your covariates, which
% might help you to assess their clinical relevance.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VPC
% At certain stages in the model building it is useful to assess the
% descriptive power of the model by generating a VPC. Since Monolix has 
% inbuild functionality for that, I am not going to recode it. 
% The user can at any time open Monolix and generate a VPC for a model of
% interest.
% As soon as Monolix allows to script the generation of VPCs easily, this
% will be implemented in the SBPOP popPKPD toolbox.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robustness check
% It is useful to assess the robustness of an estimation at certain stages
% of the model building. This can be done after selection of the base
% model, after selection of the covariance model, after selection of the
% full covariate model, and after selection of the final model.
%
% What is meant by this is simply to do many estimations of the same model
% but from different initial guesses. The function
% SBPOPbuildPopPKModelSpace allows also to do that. The only difference is
% that in this case no fit analysis plots will be generated, just the text
% files.
%
% What you would like to see after you have run this function is that the
% parameter estimates are reasonably robust to different starting
% conditions.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modeltest                           = [];                    
modeltest.numberCompartments        = 2;                     
modeltest.errorModels               = 'comb1';               
modeltest.saturableClearance        = 1;                     
modeltest.lagTime                   = 0;                     

modeltest.FACTOR_UNITS              = 1;

%                                       F,    CL,   Vc,   Q1,   Vp1,  Q2,  Vp2,  VMAX,   KM,   ka_input2, Tlag_input2
modeltest.POPvalues0                = [ 0.41    0.4     3       0.93    4.66    0.93    4.66    4.4     1.7     0.25    0.5 ];
modeltest.POPestimate               = [ 1       1       1       1       1       1       1       1       1       1       1   ];
modeltest.IIVestimate               = {[0       1       1       1       1       1       1       1       0       0       0   ]};
modeltest.IIVvalues0                = [0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5  ];
 
% Define the covariances to test 
modeltest.covarianceModels          = 'diagonal';

% Define the covariances to test 
modeltest.covariateModels           = '{CL,AGE0,WT0}, {VMAX,WT0}';

% -------------------------------------------------------------------------
% Here you set the options for the repeated fits from different initial
% guesses. Important: only parameters for which both fixed effects and
% random effects are estimated will get randomized initial guesses.
%
% Ntests:               Define the number of model runs from different
%                       initial guesses 
% std_noise_setting:    Standard deviation of the noise on the parameter
%                       initial guesses. It can be interpreted as CV%/100.
% -------------------------------------------------------------------------
optionsModelSpace                   = [];
optionsModelSpace.Ntests            = 20;
optionsModelSpace.std_noise_setting = 0.5;

% -------------------------------------------------------------------------
% Do the analysis
% -------------------------------------------------------------------------
SBPOPbuildPopPKModelSpace('MODEL_4_FINAL_ROBUSTNESS', modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix, optionsModelSpace)



