%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Template for population PK analysis using SBPOP/Monolix
% CLEAN version - without to many comments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Current limitations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Covered:
% -------
% - 1,2,3 compartmental distribution models with linear distribution,
%   linear and linear+saturable elimination 
% - Infusion, bolus and first order absorption into the central
%   compartment can be present in the same dataset.
% - Lag time on first order absorption
% - Additive, proportional, additive+proportional, and exponential residual
%   error models
% - Diagonal and off diagonal correlations between random effects
% - Selected fixed and random effects can be estimated or fixed to user
%   defined values
% - Covariates, both continuous and categorical ones can be considered on
%   any parameter for which random effects are estimated
% - In each model building step you can use default settings for
%   covariates and random effect correlations
%
% Not covered:
% ------------
% - Nonlinear bioavailability will first be implemented when the next
%   release of Monolix (4.2.2) is available - before that it does not make
%   sense
% - Inter-occasion variability is not considered for the moment. Cross-over
%   studies with washout inbetween can still be handled by splitting
%   subjects in different IDs
% - Nonlinear distribution into peripheral compartments is not yet handled.
%   Need to think if this fits the Pareto principle or not ...
% - Other routes of administration than IV and 1st order absorption into
%   central compartment are not covered at the moment. Again, I am waiting
%   for the next releases of Monolix to be able to simplify my code and then
%   I will implement additional routes (0 order absorption and maybe even
%   mixed things).
% 
% Performance limitations:
% ------------------------
% - If both IV and 1st order absorption administration is present in the
%   dataset, an ODE based solution is used, rather than analytic solutions.
%   This even if the underlying structural PK model is linear. This is not
%   nice, but will be changed as soon as Monolix 4.2.3 is out.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clc; clear all; close all; restoredefaultpath();
path2SBPOP   	= '~/TOOLS/SBPOP_Rev_1012';
path2Monolix    = '/CHBS/apps/Monolix/latest/Matlab/matlab';

oldpath 		= pwd; cd(path2Monolix);    addpath(genpath(pwd));  run_init();     cd(oldpath);
oldpath 		= pwd; cd(path2SBPOP);      installSBPOPpackage();                  cd(oldpath);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Definition of general settings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

originalDatasetFile                 	= '../dataEXAMPLE.csv'; 

covNames                            	= {'AGE0'    'HT0'      'WT0'     'BMI0'};
catNames                            	= {'SEX'     'RACE'     'ETHN'};

catImputationValues                 	= [  1         2          1];

analysisDatasetFile                 	= '../Data/popPK.csv';

optionsMonolix                          = [];                      
optionsMonolix.algorithm.SEED           = 123456;                  
optionsMonolix.algorithm.K1             = 1000;                    
optionsMonolix.algorithm.K2             = 200;                     
optionsMonolix.algorithm.K1_AUTO        = 0;                       
optionsMonolix.algorithm.K2_AUTO        = 0;                       
optionsMonolix.algorithm.NRCHAINS       = 1;                       
optionsMonolix.LLsetting                = 'importantSampling';         

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% If "Parallel Computing Toolbox" available on your system can 
% get your desired number of nodes here and in the following estimation
% runs will be done in parallel.
% Note that the code below works only for the Novartis system I use. 
% For any other system you need to make sure you get the number of nodes 
% you want (ask your sysadmin and Mathworks). The only additional thing you
% need to do is to define the path2MonolixStandalone variable with the 
% path to standalone Monolix.
% If parallel computing toolbox not available, remove this whole section
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

path2SBPAR      		= '~/TOOLS/SBPAR_Rev_1004';
oldpath 				= pwd; cd(path2SBPAR); installSBPAR(); cd(oldpath);

NumberNodes             = 10;
optionsMonolix.PathMonolixStandalone  = '/CHBS/apps/Monolix/4.2.0/Standalone/Monolix.4.2.0-standalone2008b-linux64/bin/Monolix.sh';

% Get nodes for parallel processing
SBPOPinitializeParallelComputingMODESIM(NumberNodes);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load original dataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

data = SBPOPloadCSVdataset(originalDatasetFile);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Check original dataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SBPOPcheckDataFormat(data)  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% General graphical exploration of data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

optionsGraphicalExploration                     = [];
optionsGraphicalExploration.color               = 1;
optionsGraphicalExploration.outputPath          = '../Output/DataExploration/';

optionsGraphicalExploration.plotIndividualData  = 1;
optionsGraphicalExploration.plotStratifiedData  = 1;
optionsGraphicalExploration.plotCovariateData   = 1;

SBPOPexplorePopPKdata(data,covNames,catNames,optionsGraphicalExploration)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Data Cleaning 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

removeID  = {
    1   , 'Removed subject 1 as example'
    16  , 'Removed another subject as example'
};

removeREC = {
     27   , 'Removed record for example purpose only, not due to data inspection'
     126  , 'Removed record for example purpose only, not due to data inspection'
};

Nobs = 1;

optionsDataCleaning                 = [];
optionsDataCleaning.outputPath      = '../Output/DataCleaning/';
optionsDataCleaning.removeBLLOQ     = 1;

datanew = SBPOPcleanPopPKdata(data,removeID,removeREC,Nobs,covNames,catNames,catImputationValues,optionsDataCleaning);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Conversion of data to popPK analysis dataset 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dataheaderMonolix = SBPOPconvert2popPKdataset(datanew,covNames,catNames,analysisDatasetFile);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% MODEL BUILDING
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Initial parameter guesses and other settings
% - Create single "buest guess" Monolix project and start Monolix GUI to 
%   "Check initial guesses"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modeltest                           = [];                               
modeltest.numberCompartments        = 2;                            
modeltest.errorModels               = 'comb1';  
modeltest.saturableClearance        = 1;         
modeltest.lagTime                   = 0;                              
modeltest.FACTOR_UNITS              = 1;
%                                       F,    CL,   Vc,   Q1,   Vp1,  Q2,  Vp2,  VMAX,   KM,   ka_input2, Tlag_input2
modeltest.POPvalues0                = [ 0.5   0.2   2      1     4     1   2     0.5     5     0.2        0.5];
modeltest.POPestimate               = [  1     1    1      1     1     1   1      1      1      1           1];
modeltest.IIVestimate               = [  1     1    1      1     1     1   1      1      1      1           1];

optionsModelSpace                   = [];
optionsModelSpace.buildModelsOnly   = 1;

SBPOPbuildPopPKModelSpace('MODEL_0_TEST', modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix, optionsModelSpace);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Base model building
% - Random effects on all parameters
% - Diagonal random effect correlation matrix
% - Taking into account covariates possible, if desired
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modeltest                           = [];                             
modeltest.numberCompartments        = [2 3];                          
modeltest.errorModels               = {'const','prop','comb1'};        
modeltest.saturableClearance        = [0 1];                   
modeltest.lagTime                   = [0 1]; 
modeltest.FACTOR_UNITS              = 1;
%                                       F,    CL,   Vc,   Q1,   Vp1,  Q2,  Vp2,  VMAX,   KM,   ka_input2, Tlag_input2
modeltest.POPvalues0                = [ 0.41    0.4     3       0.93    4.66    0.93    4.66    4.4     1.7     0.25    0.5 ];
modeltest.POPestimate               = [ 1       1       1       1       1       1       1       1       1       1       1   ];
modeltest.IIVestimate               = [ 1       1       1       1       1       1       1       1       1       1       1   ];
modeltest.IIVvalues0                = [0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5  ];

SBPOPbuildPopPKModelSpace('MODEL_1_BASE', modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Covariance model building
% - Start with full correlation matrix
% - Potentially remove correlations including parameters with high shrinkage
% - If "few" subjects in the dataset, consider also smaller blocks, e.g, {CL,Vc} 
% - Let selection of covariate model be followed by a robustness assessment step
%   to assess identifiability of random effect correlations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modeltest                           = [];                   
modeltest.numberCompartments        = [2 3];                
modeltest.errorModels               = {'comb1'};            
modeltest.saturableClearance        = [1];                         
modeltest.lagTime                   = [0];                         
modeltest.FACTOR_UNITS              = 1;
%                                       F,    CL,   Vc,   Q1,   Vp1,  Q2,  Vp2,  VMAX,   KM,   ka_input2, Tlag_input2
modeltest.POPvalues0                = [ 0.41    0.4     3       0.93    4.66    0.93    4.66    4.4     1.7     0.25    0.5 ];
modeltest.POPestimate               = [ 1       1       1       1       1       1       1       1       1       1       1   ];
modeltest.IIVestimate               = {[1       1       1       1       1       1       1       1       1       1       1   ]
                                       [0       1       1       1       1       1       1       1       1       0       0   ]
                                       [0       1       1       1       1       1       1       1       0       0       0   ]};
modeltest.IIVvalues0                = [0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5  ];

modeltest.covarianceModels          = {'diagonal', ...
                                       '{CL,Vc}', ... 
                                       '{CL,Vc,Q1,Vp1}', ... 
                                       '{CL,Vc},{VMAX,KM}', ... 
                                       '{CL,Vc,Q1,Vp1},{VMAX,KM}', ... 
                                       };

SBPOPbuildPopPKModelSpace('MODEL_2_COVARIANCE', modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Covariate model building
% 1) Pre-specify candidate covariates to test in modeling plan  
% 2) Add all covariates to test on all parameters you are interested in
%    You might want to exclude simultaneous testing of highly correlated 
%    covariates on the same parameter at the same time
% 3) Retain statistically significant covariates (p-value or 
%    SBPOPcovariateChangeAssessment (see below) and refit the covariate 
%    coefficients
% 4) Fix the previously found covariate coefficients, and test correlated 
%    candidate covariates (e.g. Japanese vs. non Japanese for an ethnic 
%    sensitivity analysis) to assess their significance
% 5) Keep the covariate model with statistically significant covariates 
%    and assess their clinical relevance (see below)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modeltest                           = [];                       
modeltest.numberCompartments        = [2];                      
modeltest.errorModels               = {'comb1'};                
modeltest.saturableClearance        = [1];                      
modeltest.lagTime                   = [0];                      
modeltest.FACTOR_UNITS              = 1;

%                                       F,    CL,   Vc,   Q1,   Vp1,  Q2,  Vp2,  VMAX,   KM,   ka_input2, Tlag_input2
modeltest.POPvalues0                = [ 0.41    0.4     3       0.93    4.66    0.93    4.66    4.4     1.7     0.25    0.5 ];
modeltest.POPestimate               = [ 1       1       1       1       1       1       1       1       1       1       1   ];
modeltest.IIVestimate               = {[0       1       1       1       1       1       1       1       0       0       0   ]};
modeltest.IIVvalues0                = [0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5  ];

modeltest.covarianceModels          = {'diagonal', ...
                                       '{CL,Vc}'
                                       };

modeltest.covariateModels           = {'',...
                                       '{CL,AGE0,WT0}, {Vc,AGE0,WT0}, {VMAX,AGE0,SEX}', ...
                                       '{CL,AGE0,WT0}, {Vc,AGE0,WT0}, {VMAX,AGE0,WT0}', ...
                                       '{CL,AGE0,WT0}, {Vc,AGE0,WT0}, {VMAX,AGE0,WT0,SEX}'
                                       };

SBPOPbuildPopPKModelSpace('MODEL_3_COVARIATE', modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Covariate effects on parameters
% - Check how much the identified covariate relationships do change the 
%   parameter values.
% - Prespecify in your modeling plan a cut-off value above which you 
%   consider a covariate to be "clinically relevant". 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

FIT_PATH = '../Models/MODEL_3_COVARIATEmodel/FITMODEL_3_COVARIATE_004';

SBPOPcovariateChangeAssessment(FIT_PATH, analysisDatasetFile)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Simulation approach to investigate clinical relevance of covariates
% - Better approach than "cut-off value" above
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% VPC
% Use Monolix GUI to generate VPCs for now
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robustness check
% - Useful at several stages of model development. Use as appropriate.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

modeltest                           = [];                    
modeltest.numberCompartments        = 2;                     
modeltest.errorModels               = 'comb1';               
modeltest.saturableClearance        = 1;                     
modeltest.lagTime                   = 0;                     
modeltest.FACTOR_UNITS              = 1;

%                                       F,    CL,   Vc,   Q1,   Vp1,  Q2,  Vp2,  VMAX,   KM,   ka_input2, Tlag_input2
modeltest.POPvalues0                = [ 0.41    0.4     3       0.93    4.66    0.93    4.66    4.4     1.7     0.25    0.5 ];
modeltest.POPestimate               = [ 1       1       1       1       1       1       1       1       1       1       1   ];
modeltest.IIVestimate               = {[0       1       1       1       1       1       1       1       0       0       0   ]};
modeltest.IIVvalues0                = [0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5     0.5  ];

modeltest.covarianceModels          = 'diagonal';

modeltest.covariateModels           = '{CL,AGE0,WT0}, {VMAX,WT0}';

optionsModelSpace                   = [];
optionsModelSpace.Ntests            = 20;
optionsModelSpace.std_noise_setting = 0.5;

SBPOPbuildPopPKModelSpace('MODEL_4_FINAL_ROBUSTNESS', modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix, optionsModelSpace)


