function [] = SBPOPcheckDataFormat(data)
% [DESCRIPTION]
% The SBPOP popPKPDtoolbox assumes a standard PKPD dataset specification.
% This function here will check the availability of the required columns.
% Additionally, it will do some sanity checks of the dataset and report 
% potential issues in the Command Window as text.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Standard Dataset Specification
% I am aware that some of the column names are "strange" but historic
% reasons led to them. Even now in 2013 programmers told me: 4 characters
% is max. So TIM2 is one of these constructs which is standard in my
% current work - I did not decide these names, but I clearly put in the
% longer column names. If you are unhappy with these names, just change
% them as you wish, but do so in all scripts where they are used ;-)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   STUD:       Numeric study number   
%   SUBJ:       Numeric subject identifier
%   ID:         Unique subject identifier (to be used for model building)
%   TYPE:       Column with type identifier for each record 
%               Standard assumes dose record has TYPE 0 and PK observation
%               record has TYPE 1
%   TIM2:       Time post first dose (negative values for observation
%               records pre-first dose)
%   TAD:        Time since last dose (pre-first-dose values same as TIM2
%               values)
%   TIMEUNIT:   Column with units of time measurements in dataset
%   DV:         Observation value (0 for dose records, LOQ for values above
%               or below the limit of quantification if CENS not equal 0)
%   UNIT:       Column with units for record entries
%   NAME:       Column with names/labels of record entries
%   MDV:        Missing data value columns (0 if observation value is
%               defined, 1 for dose records and for unknown observation
%               values)
% 	EVID: 		Event ID. 0 for observations, 1 for dosing records.
%   AMT:        Dose given at dosing instant (0 for observation records).
%               Placebo subjects need AMT=0 at times of placebo
%               administration
%   ADM:        Administration column (0 for observation records, 1 for IV
%               infusion or bolus, 2 for first order absorption into
%               central compartment)  
%   DOSE:       This column contains the value of the last dose given. 
%               It is used for dose-normalization of the concentration. 
%   TRT:        Numeric treatment assignment. In case of fixed dose dosing
%               the column contains the same information as the DOSE
%               column. Differences will appear if dosing defined in mg/kg
%               or mg/m2, etc. Then the TRT column will contain the mg/kg
%               or mg/m2 values, while the DOSE column will contain the
%               actual dose levels given to a specific patient.
%               This column is used for stratification of the graphical
%               exploration plots. For placebo subjects TRT=0 should be
%               chosen.
%   REGIMEN:    Numeric regimen assignment. Different regimen should obtain
%               different numeric identifiers. This column is used for
%               stratification of the graphical exploration plots. 
%   CENS:       Censored data (=1 for observation records below lower limit
%               of quantification, =-1 for observation records above upper
%               limit of quantification, =0 for all other records)  
%
% Not required columns, which are used when defined are:
%
%   TINF:       Infusion time. (0 for observation records, 0 for "Bolus" or
%               "first order absorption" dose records, Infusion time in
%               TIMEUNIT unit if infusion dose record).
%
% Additional columns:
%
% - User defined continuous and categorical covariates. Here only time
%   independent covariates should be considered.
%   -----------------------------------------------------------------------
%   This could/should include variables such as disease related biomarkers,
%   since the PK might depend on the PD, concomittant medications, etc.
%   -----------------------------------------------------------------------
% - User defined time dependent covariates. These could be used for users
%   own graphical analysis. "Automatic" generation of exploratory plots for
%   these currently not supported.
%
% Columns that might make it into the standard:
% (They are no in yet - but make sense - just putting them here to remind
% myself)
%
% - VISIT:      Visit number
% - BASE:       Baseline flag. =0 for non baseline observation records, 
%               =1 for baseline observation records, =0 for dose records
% - SCREEN:     Screening flag. =0 for non screening observation records, 
%               =1 for screening observation records, =0 for dose records
% - EVDT:       Date/time string for this record.
% - AENAME:     String with adverse event label
% - AEDURATION: Duration of observed adverse event in dataset time unit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% [SYNTAX]
% [] = SBPOPcheckDataFormat(data)
%
% [INPUT]
% data:         MATLAB dataset containing the dosing and observation
%               information to be displayed. 
%
% [OUTPUT]
% If at least one of the required columns is not present an error will be
% shown.
%
% [ASSUMPTIONS]
%
% [AUTHOR]
% Henning Schmidt, henning.schmidt@novartis.com
%
% [DATE]
% 24th February 2013
%
% [PLATFORM]
% Windows, Unix, MATLAB
%
% [TOOLBOXES USED]
% Statistics Toolbox

% Information:
% ============
% Copyright (c) 2012 Novartis Pharma AG
% 
% This program is Free Open Source Software: you can redistribute it and/or modify 
% it under the terms of the GNU General Public License as published by 
% the Free Software Foundation, either version 3 of the License, or 
% (at your option) any later version. 
% 
% This program is distributed in the hope that it will be useful, 
% but WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License 
% along with this program. If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check input arguments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isa(data,'dataset'),
    error('Input argument is not a MATLAB dataset.');
end
datanames = get(data,'VarNames');
requiredColumns = {'ID' 'TYPE' 'TIM2' 'DV' 'AMT' 'TIMEUNIT' 'NAME' 'UNIT' 'MDV' 'EVID' 'STUD' 'TRT' 'REGIMEN' 'SUBJ' 'TAD' 'ADM' 'CENS' 'DOSE'};
errorText = '';
for k=1:length(requiredColumns),
    ix = strmatchSB(requiredColumns{k},datanames,'exact');
    if isempty(ix), 
        errorText = sprintf('%sThe dataset does not contain the column ''%s''.\n',errorText,requiredColumns{k});  %#ok<*SPERR>
    end    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check if SS is present => error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(strmatchSB('SS',upper(datanames))),
    errorText = sprintf('%sColumn "SS" is present in the dataset. This is outdated, please consider the use of "ADDL" in Monolix.',errorText);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Show error if needed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(errorText),
    error(errorText);
end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Do additional checks
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If EVID column contains non-zero elements then check that each subject 
% has received a dose (even placebo subjects - placebo dose of 0 then).
% If not, then ERROR!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(data.EVID==1) ~= 0,
    allID = unique(data.ID);
    errorText = '';
    for k=1:length(allID),
        datak = data(data.ID==allID(k),:);
        if sum(datak.EVID==1) == 0,
            errorText = sprintf('%sID=%d (STUD=%d) does not have any dose records. Please include even for placebo subjects (AMT=0).\n',errorText,allID(k),datak.STUD(1));
        end
    end   
    if ~isempty(errorText),
        error(errorText);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check uniqueness of TIM2 per ID and TYPE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allID = unique(data.ID);
for k=1:length(allID),
    datak = data(data.ID==allID(k),:);
    allTYPE = unique(datak.TYPE);
    for k2=1:length(allTYPE),
        if allTYPE(k2) ~= 0,
            % Check only for observation records!
            datakk2 = datak(datak.TYPE==allTYPE(k2),:);
            % Get TIM2
            TIM2 = datakk2.TIM2;
            % Check it
            if ~(length(TIM2) == length(unique(TIM2))),
                fprintf('ID=%d has observation records of TYPE=%d at same time points.\n',allID(k),allTYPE(k2));
            end
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check monotonous non decreasing TIM2 per ID
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allID = unique(data.ID);
for k=1:length(allID),
    datak = data(data.ID==allID(k),:);
    if sum(diff(datak.TIM2) < 0),
        fprintf('TIM2 non monotonously increasing for ID=%d.\n',allID(k));
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check uniqueness of TIMEUNIT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
timeunits = unique(data.TIMEUNIT);
if length(timeunits) > 1,
    fprintf('Different time units are present in the TIMEUNIT column.\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check uniqueness of NAME and UNIT per TYPE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allTYPE = unique(data.TYPE);
for k=1:length(allTYPE),
    datak = data(data.TYPE==allTYPE(k),:);
    % Check NAME
    names = unique(datak.NAME);
    if length(names) ~= 1,
        fprintf('Different entries for TYPE "%d" are present in the NAME column.\n',allTYPE(k));
    end
    % Check UNIT
    unit = unique(datak.UNIT);
    if length(unit) ~= 1,
        fprintf('Different entries for TYPE "%d" are present in the UNIT column.\n',allTYPE(k));
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in TIM2
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.TIM2)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the TIM2 column. If time undefined in clinical database, don''t include this record into the dataset.\n');
end    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in TAD 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.TAD)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the TAD column. For pre-FIRST-dose samples set TAD=TIM2\n');
end    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in DV 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.DV)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the DV column. If undefined in clinical database, don''t include this record into the dataset. For dose records use "0".\n');
end   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in TYPE 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.TYPE)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the TYPE column.\n');
end   

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in ID
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.ID)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the ID column.\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in AMT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.AMT)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the AMT column. For observation records, use "0".\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in ADM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.ADM)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the ADM column. For observation records, use "0".\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in MDV
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.MDV)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the MDV column.\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in STUD
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.STUD)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the STUD column.\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in SUBJ
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.SUBJ)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the SUBJ column.\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in DOSE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.DOSE)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the DOSE column.\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in TRT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.TRT)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the TRT column.\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in REGIMEN
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if sum(isnan(data.REGIMEN)) > 0,
    fprintf('Undefined (NaN or empty) values are present in the REGIMEN column.\n');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check that TRT and REGIMEN is unique in each ID
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allID = unique(data.ID);
for k=1:length(allID),
    datak = data(data.ID==allID(k),:);
    % Check TRT
    if length(unique(datak.TRT)) ~= 1,
        fprintf('Different entries for TRT in subject "%d" are present.\n',allID(k));
    end
    % Check REGIMEN
    if length(unique(datak.REGIMEN)) ~= 1,
        fprintf('Different entries for REGIMEN in subject "%d" are present.\n',allID(k));
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check NaN in TINF if TNIF column available
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(strmatchSB('TINF',datanames)),
    if sum(isnan(data.TINF)) > 0,
        fprintf('Undefined (NaN or empty) values are present in the TINF column. For observation records, use "0".\n');
    end
end

