function [datanew] = SBPOPcleanPopPKdata(data,removeID,removeREC,Nobs,covNames,catNames,catImputationValues,options)
% [DESCRIPTION]
% This function is a wrapper for different cleaning functions. Calling 
% - SBPOPcleanRemoveRecordsIDs
% - SBPOPcleanRemovePlaceboSubjects
% - SBPOPcleanRemoveFewObsSubjects
% - SBPOPcleanImputeCovariates
%
% Additionally, by default BLLOQ data (defined by CENS=1) are removed from
% the dataset. But only here for the popPK. If you want to keep the data,
% please set the optional argument "options.removeBLLOQ=0".
%
% The data need to be provided, following the standard dataspec, defined in
% the help to the function SBPOPcheckDataFormat, so please look there for
% more information.   
% 
% [SYNTAX]
% [datanew] = SBPOPcleanPopPKdata(data,removeID,removeREC,nObs,covNames,catNames,catImputationValues)
% [datanew] = SBPOPcleanPopPKdata(data,removeID,removeREC,nObs,covNames,catNames,catImputationValues,options)
%
% [INPUT]
% data:         MATLAB PKPD dataset in standard data spec format  
% removeID:     Cell-matrix with 2 columns. First column contains the IDs
%               of the subjects to be removed from the dataset. The second
%               column contains strings, which define the reason why this
%               ID is removed.
% removeREC:    Cell-matrix with 2 columns. First column contains the indices
%               of the records to be removed from the dataset. The second
%               column contains strings, which define the reason why this
%               record is removed.
% Nobs:         All subjects from the dataset which do have <= Nobs
%               observation records from TYPE=1 (PK observations) with MDV=0 are removed.
% covNames:     Cell-array with names of continuous covariates
% catNames:     Cell-array with names of categorical covariates
% catImputationValues:     Vector with same length as catNames, specifying
%               the imputation values for these categorical covariates (if
%               needed)
% options:      MATLAB structure with additional options
%
%               options.removeBLLOQ: =0: keep BLLOQ data, =1: remove BLLOQ
%                                    data (default: 1)
%               options.outputPath:  path where
%                                    outputs are exported to. Default:
%                                    '../Output/DataCleaning/';
%
% [OUTPUT]
% datanew:      cleaned dataset 
%
% [ASSUMPTIONS]
%
% [AUTHOR]
% Henning Schmidt, henning.schmidt@novartis.com
%
% [DATE]
% 9th May 2010
%
% [PLATFORM]
% Windows XP Engine, MODESIM, MATLAB R2009a
%
% [KEYWORDS]
% MATLAB, SBPOP, dataexploration, datacleaning
% 
% [TOOLBOXES USED]
% Statistics Toolbox
%
% [VALIDATION HISTORY]
%
% [MODIFICATION HISTORY]

% Information:
% ============
% Copyright (c) 2012 Novartis Pharma AG
% 
% This program is Free Open Source Software: you can redistribute it and/or modify 
% it under the terms of the GNU General Public License as published by 
% the Free Software Foundation, either version 3 of the License, or 
% (at your option) any later version. 
% 
% This program is distributed in the hope that it will be useful, 
% but WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License 
% along with this program. If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check input arguments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SBPOPcheckDataFormat(data);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Handle options
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
try outputPath  = [options.outputPath '/']; catch, outputPath = '../Output/DataCleaning/';  end; %#ok<*CTCH>
try removeBLLOQ = options.removeBLLOQ;      catch, removeBLLOQ = 1;                         end; %#ok<*CTCH>

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create output folder
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[p,f,e] = fileparts(outputPath);
warning off
mkdir(p);
warning on

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run SBPOPcleanRemoveRecordsIDs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
filename = [outputPath '01_Data_Cleaning_Manual.txt'];
datanew = SBPOPcleanRemoveRecordsIDs(data,removeID,removeREC,filename);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run SBPOPcleanRemovePlaceboSubjects
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
filename = [outputPath '02_Data_Cleaning_Placebo_Subjects.txt'];
datanew = SBPOPcleanRemovePlaceboSubjects(datanew,filename);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run SBPOPcleanRemoveFewObsSubjects
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
type = 1; % PK is TYPE=1
filename = [outputPath '03_Data_Cleaning_Few_Observations.txt'];
datanew = SBPOPcleanRemoveFewObsSubjects(datanew,Nobs,type,filename);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run SBPOPcleanImputeCovariates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
filename = [outputPath '04_Covariate_Imputation.txt'];
datanew  = SBPOPcleanImputeCovariates(datanew,covNames,catNames,catImputationValues,filename);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove BLLOQ data if desired
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if removeBLLOQ,
    % Get the BLOQ data
    dataBLLOQ = datanew(datanew.CENS==1,:);
    % Remove the BLOQ data
    datanew(datanew.CENS==1,:) = [];
    % Save the removed data in the output path
    datafilenameBLLOQ = [outputPath '05_removed_BLLOQ_data.csv'];
    SBPOPexportCSVdataset(dataBLLOQ,datafilenameBLLOQ);
    disp('The following BLOQ data were removed:');
    disp('=====================================');
    dataBLLOQ
end
