function [datanew] = SBPOPcleanRemovePlaceboSubjects(data,filename)
% [DESCRIPTION]
% This function simply removes all records for which TRT==0. It is checked
% that if TRT==0 then it is 0 for the whole subject. Otherwise a warning is
% done. If a filename is provided, then a log of the changes in the dataset
% is exported to this file.
% 
% The data need to be provided, following the standard dataspec, defined in
% the help to the function SBPOPcheckDataFormat, so please look there for
% more information.   
%
% [SYNTAX]
% [datanew] = SBPOPcleanRemovePlaceboSubjects(data)
% [datanew] = SBPOPcleanRemovePlaceboSubjects(data,filename)
%
% [INPUT]
% data:         MATLAB PKPD dataset in standard data spec format  
% filename:     String with filename / path for export of information in
%               same format as displayed in command window. If not defined,
%               then no file will be created.
%
% [OUTPUT]
% datanew:      dataset with removed placebo (TRT==0) subjects. 
%
% [ASSUMPTIONS]
% The input dataset needs to follow the SBPOP specification for clinical data. 
%
% [AUTHOR]
% Henning Schmidt, henning.schmidt@novartis.com
%
% [DATE]
% 9th May 2010
%
% [PLATFORM]
% Windows XP Engine, MODESIM, MATLAB R2009a
%
% [KEYWORDS]
% MATLAB, SBPOP, dataexploration, datacleaning
% 
% [TOOLBOXES USED]
% Statistics Toolbox
%
% [VALIDATION HISTORY]
%
% [MODIFICATION HISTORY]

% Information:
% ============
% Copyright (c) 2012 Novartis Pharma AG
% 
% This program is Free Open Source Software: you can redistribute it and/or modify 
% it under the terms of the GNU General Public License as published by 
% the Free Software Foundation, either version 3 of the License, or 
% (at your option) any later version. 
% 
% This program is distributed in the hope that it will be useful, 
% but WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License 
% along with this program. If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Handle optional input argument
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin==1,
    filename = '';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check input arguments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SBPOPcheckDataFormat(data);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get info about placebo subjects and create datanew dataset without them
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
placeboIDs = [];
allID = unique(data.ID);
datanew = dataset();
for k=1:length(allID),
    datak = data(data.ID==allID(k),:);
    TRTk = datak.TRT;
    if sum(abs(TRTk)) == 0,
        placeboIDs = [placeboIDs allID(k)];
    else
        datanew = [datanew; datak];
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Prepare user info text about placebo patients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create text output
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
text = '';
text = sprintf('The function "SBPOPcleanRemovePlaceboSubjects" has removed the following %d subjects (IDs shown):\n',length(placeboIDs));
text = sprintf('%s(Settings: removing subjects with TRT column all 0)\n',text);
text = sprintf('%s=================================================================================================\n',text);
count = 1;
textID = '';
for k=1:length(placeboIDs),
    textID = sprintf('%s%8d ',textID,placeboIDs(k));
    count = count+1;
    if count == 12,
        textID = sprintf('%s\n',textID);
        count = 1;
    end
end
if isempty(placeboIDs),
    textID = sprintf('No placebo subjects present.');
end
text = sprintf('%s%s\n',text,textID);
text = sprintf('%s=================================================================================================\n',text);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Show output in command window
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(text);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Export results to file if filename defined
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(filename),
    % Create output folder if not yet existing
    [p,f,e] = fileparts(filename);
    warning off
    mkdir(p);
    warning on
        
    fid = fopen([strrep(filename,'.txt','') '.txt'],'w');
    fprintf(fid,'%s\n\n',text);
    fclose(fid);
end
