function [dataheaderMonolix,dataheaderMonolix_PLACEBO] = SBPOPconvert2popPDconcentrationDataset(model,PKvariableName,deltaTIMEConcentrationSimulation,dosing,path_to_monolix_project_folder,data,TYPE,covNames,catNames,analysisDataset)
% [DESCRIPTION]
% This function converts the given dataset in "data" into a popPD dataset
% where the individual concentration is defined in a column, to be used as
% a regression parameter.
% The datapopPD dataset will contain a subset of the columns in the
% original dataset, sufficient to perform a popPD analysis. 
% The dataset will only contain PD observation records of type "TYPE", set
% to YTYPE=1. Additional records of TYPE=1 are added but with MDV=2 to
% allow passing of updated concentration values in the regression variable.
% 
% The data need to be provided, following the standard dataspec, defined in
% the help to the function SBPOPcheckDataFormat, so please look there for
% more information.   
%
% Two datasets will be created. One with all subjects and one with only
% Placebo subjects.
%
% These concentration datasets are not that useful for placebo model
% building. The reason is that the goal is to speed up the PD estimation
% process by not needing to apply doses and reset the integrator. If
% placebo doses would be added then dosing would be back and speed would be
% lost. If needed later we can rethink this ...
%
% [SYNTAX]
% [dataheaderMonolix,dataheaderMonolix_PLACEBO] = SBPOPconvert2popPDparametersDataset(model,dosing,path_to_monolix_project_folder,data,TYPE,covNames,catNames,analysisDataset)
%
% [INPUT]
% model:        SBmodel: PKPD model to be used for simulation / estimation
%               - only used to check the parameters in the fit results. 
% PKvariableName: Name of the PK variable in the provided model (to read
%                 out the simulated information)
% deltaTIMEConcentrationSimulation: Time interval in which to determine the
%                 concentration and add to the PD dataset. Additionally,
%                 the same times as the PK and PD observations are used.
% dosing:       SBPOPdosing scheme that will be used for simulation.
% path_to_monolix_project_folder: Path to the Monolix PK fit to get the
%                                 individual parameters from 
% data:         MATLAB PKPD dataset in standard data spec format  
% TYPE:         The numeric type identifier (column TYPE), specifying which
%               PD readout is to be modeled and kept in the PD dataset
% covNames:     Cell-array with names of continuous covariates
% catNames:     Cell-array with names of categorical covariates
% analysisDataset:    Filename, including path for saving the popPD dataset
%                     as CSV file. 
%
% [OUTPUT]
% dataheaderMonolix:    Data header for Monolix
% dataheaderMonolix_PLACEBO:    Removed "X" and set to IGNORE since the
%                               placebo models only will not contain
%                               regression variables 
%                               Additionally, removed EVID and AMT by
%                               setting to ignore since not needed and
%                               otherwise Monolix will not work with it.
%
% [ASSUMPTIONS]
%
% [AUTHOR]
% Henning Schmidt, henning.schmidt@novartis.com
%
% [DATE]
% 9th May 2010
%
% [PLATFORM]
% Windows XP Engine, MODESIM, MATLAB R2009a
%
% [KEYWORDS]
% MATLAB, SBPOP, dataexploration, datacleaning
% 
% [TOOLBOXES USED]
% Statistics Toolbox
%
% [VALIDATION HISTORY]
%
% [MODIFICATION HISTORY]

% Information:
% ============
% Copyright (c) 2012 Novartis Pharma AG
% 
% This program is Free Open Source Software: you can redistribute it and/or modify 
% it under the terms of the GNU General Public License as published by 
% the Free Software Foundation, either version 3 of the License, or 
% (at your option) any later version. 
% 
% This program is distributed in the hope that it will be useful, 
% but WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License 
% along with this program. If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define some reasonable options for simulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
optionsSimulation = [];
optionsSimulation.abstol = 1e-6;
optionsSimulation.reltol = 1e-6;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get population parameters (for placebo subjects)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
param                   = SBPOPsampleMONOLIXparam(path_to_monolix_project_folder,0,0);
ParameterNamesFIT       = param.parameterNames;
PopParameterValues      = param.parameterValuesPopulation;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check that the parameters  in the fit result also are in the model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
moddos                  = mergemoddosSBPOP(model,dosing);
paramModdos             = SBparameters(moddos);
IXparametersNotInModel  = [];
for k=1:length(ParameterNamesFIT),
    ix = strmatchSB(ParameterNamesFIT{k},paramModdos,'exact');
    if isempty(ix),
        warning(sprintf('Parameter "%s" available in the fit results but not in the PK model. This parameter will be neglected.\n\tPlease make sure that this does make sense!',ParameterNamesFIT{k}));
        IXparametersNotInModel(end+1) = k;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get individual PK parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
indiv_param             = SBPOPloadNONCSVdataset([path_to_monolix_project_folder '/RESULTS/indiv_parameters.txt']);
indiv_param             = indiv_param(:,1:length(ParameterNamesFIT)+1);
% Remove parameters that are not in the model
indiv_param(:,IXparametersNotInModel+1) = [];
% Remove the _mode thing
indiv_param = set(indiv_param,'VarNames',strrep(get(indiv_param,'VarNames'),'_mode',''));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove parameters from the names and population things if needed (after having gotten the individual PK parameters)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ParameterNamesFIT(IXparametersNotInModel) = [];
PopParameterValues(IXparametersNotInModel) = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Store PK observation data for later comparison etc.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dataPK = data(data.TYPE==1,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove all but dose and PD of interest records from the dataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
data(data.TYPE~=0 & data.TYPE~=TYPE,:) = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create dataset with simulated PK concentrations 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Create dataset and add concentration column
% For placebo subjects the time points of the PD observations is
% sufficient. For the others additional time points will be added
dataPKconc = data(data.TYPE==TYPE,:);
dataPKconc.PKconcentration = zeros(length(dataPKconc),1);

% Get dose only records
dataDOSE = data(data.TYPE==0,:);

% Create mexModel
SBPDmakeMEXmodel(moddos,'mexModel');

% Simulate and build dataset
allID = unique(dataPKconc.ID);
dataPKconcFilled = dataset();
disp('Simulating the individual concentrations ...');
parfor k=1:length(allID),
    % Get individual PD information (needed for time)
    dataOBSk = dataPKconc(dataPKconc.ID==allID(k),:);
    % Get individual dosing information
    datak = dataDOSE(dataDOSE.ID==allID(k),:);
    % Simulate only if dose available
    if ~isempty(datak),
        % Get dosing info for IV and ABS1
        TIME_IV = datak.TIM2(datak.ADM==1);
        DOSE_IV = datak.AMT(datak.ADM==1);
        TINF_IV = datak.TINF(datak.ADM==1);
        TIME_SC = datak.TIM2(datak.ADM==2);
        DOSE_SC = datak.AMT(datak.ADM==2);
        % Create individual dosing scheme
        ds = struct(dosing);
        if length({ds.inputs.type}) ~= length(unique({ds.inputs.type})),
            error('Same type of dose administration happens more than once in the provided dosing scheme. Can not be handled!');
        end
        % Handle INFUSION first
        ix = strmatchSB('INFUSION',{ds.inputs.type});
        if ~isempty(ix),
            if ~isempty(TIME_IV),
                ds.inputs(ix).time = TIME_IV(:)';
                ds.inputs(ix).D = DOSE_IV(:)';
                ds.inputs(ix).parameters.value = TINF_IV(:)';
            else
                ds.inputs(ix).time = 0;
                ds.inputs(ix).D = 0;
                ds.inputs(ix).parameters.value = 1;
            end
        end
        % Handle BOLUS next (should only happen if absorption coded in the PK model)
        % Potential lagtime will be handled by parameters in moddos
        ix = strmatchSB('BOLUS',{ds.inputs.type});
        if ~isempty(ix),
            if ~isempty(TIME_SC),
                ds.inputs(ix).time = TIME_SC(:)';
                ds.inputs(ix).D = DOSE_SC(:)';
            else
                ds.inputs(ix).time = 0;
                ds.inputs(ix).D = 0;
            end
        end
        % Handle ABSORPTION (If no dosing compartment in PK model)
        % Potential lagtime and the absorption rate constant will be handled by
        % parameters in moddos
        ix = strmatchSB('ABSORPTION1',{ds.inputs.type});
        if ~isempty(ix),
            if ~isempty(TIME_SC),
                ds.inputs(ix).time = TIME_SC(:)';
                ds.inputs(ix).D = DOSE_SC(:)';
                ds.inputs(ix).parameters.value = NaN*ones(length(TIME_SC));
            else
                ds.inputs(ix).time = 0;
                ds.inputs(ix).D = 0;
                ds.inputs(ix).parameters.value = 1;
            end
        end
        % Create dosing
        dosing_sim_indiv = SBPOPdosing(ds);
        
        % Get simulation time vector for PD observations
        TIMEOBS = dataOBSk.TIM2;
        if length(TIMEOBS) ~= length(unique(TIMEOBS)),
            % Add some random noise (very little) to handle cases where in the
            % dataset the same record is repeated with same time (does happen -
            % and crashes the simulation)
            TIMEOBS = sort(TIMEOBS+0.01*randn(length(dataOBSk),1));
        end
        
        % Get simulation time vector for additional elements (user defined + we
        % will add PK observation times if available)
        SIMTIME = [0:deltaTIMEConcentrationSimulation:max(TIMEOBS)+deltaTIMEConcentrationSimulation];
        if ~isempty(dataPK),
            OBSTIME_PK = dataPK.TIM2(dataPK.ID==allID(k));
            SIMTIME = sort(unique([SIMTIME(:)' OBSTIME_PK(:)']));
        end
        % Remove times at which PD has been measured
        SIMTIME = setdiff(SIMTIME,intersect(SIMTIME,dataOBSk.TIM2));
        
        % Get the simulation parameters
        ix = find(indiv_param.ID==allID(k));
        % If available then do the simulation, otherwise not
        if ~isempty(ix),
            indiv_param_k = indiv_param(ix,2:end);
            pnames = get(indiv_param_k,'VarNames');
            pvalues = double(indiv_param_k(1,:));
            
            try
                % Simulate once for the PD observation time points
                simresOBSTIMES = SBPOPsimdosing('mexModel',dosing_sim_indiv,TIMEOBS,[],pnames,pvalues,optionsSimulation);
                % Simulate once more for the additional time points
                simresADDTIMES = SBPOPsimdosing('mexModel',dosing_sim_indiv,SIMTIME,[],pnames,pvalues,optionsSimulation);

                SIMPK_OBSTIMES = simresOBSTIMES.variablevalues(:,variableindexSB(moddos,PKvariableName));
                SIMPK_ADDTIMES = simresADDTIMES.variablevalues(:,variableindexSB(moddos,PKvariableName));
            catch
                SIMPK_OBSTIMES = NaN(length(TIMEOBS),1);
                SIMPK_ADDTIMES = NaN(length(SIMTIME),1);
                fprintf('Error simulating ID=%d. Please check.\n',allID(k));
            end
            % Add the simulated concentrations at the obs times to the data
            dataOBSk.PKconcentration = SIMPK_OBSTIMES(:);
            
            % Add additional simulated concentrations to the dataset
            Nadd                        = length(SIMTIME);
            template                    = dataOBSk(ones(1,Nadd),:);
            template.TYPE               = TYPE*ones(Nadd,1); % Use same TYPE for additional concentration values
            template.BASE               = zeros(Nadd,1);
            template.SCREEN             = zeros(Nadd,1);
            template.TIM2               = SIMTIME(:);
            template.DV                 = zeros(Nadd,1);
            template.UNIT(1:end)        = {'.'};
            template.NAME(1:end)        = {'Dummy entry to pass PK concentration (MDV=2)'};
            template.MDV                = 1*ones(Nadd,1);  % Set to 1 since EVID=2
            template.CENS               = zeros(Nadd,1);
            template.PKconcentration    = SIMPK_ADDTIMES(:);
            template.EVID               = 2*ones(Nadd,1);  % Tells Monolix to update the regression variables
            template.AMT                = zeros(Nadd,1);   % Keep AMT column because EVID column present
            
            % Combine dataOBSk and template
            dataOBSk = [dataOBSk; template];
        end
    end
    % Combine data
    dataPKconcFilled = [dataPKconcFilled; dataOBSk];
end
% Sort data
dataPKconcFilled = sortrows(dataPKconcFilled,{'ID','TIM2','TYPE'});

% Delete mex model
clear mex
delete(['mexModel.' mexext]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compare PK observations with simulation results to check that its doing the right thing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Nrows = 4;
Ncols = 4;
[p,f,e] = fileparts(analysisDataset);
warning off
mkdir(p);
warning on
filenameCheckPlot = [p '/' f '_CHECK'];
startNewPrintFigureSBPOP(filenameCheckPlot);
if ~isempty(dataPK),
    allID = unique(dataPK.ID);
    for k=1:length(allID),
        datakPKobs_k = dataPK(dataPK.ID==allID(k),:);
        datakPKsim_k = dataPKconcFilled(dataPKconcFilled.ID==allID(k),:);
        % get time and data
        TIME_obs = datakPKobs_k.TIM2;
        TIME_sim = datakPKsim_k.TIM2;
        PK_obs = datakPKobs_k.DV;
        PK_sim = datakPKsim_k.PKconcentration;
        if mod(k-1,Nrows*Ncols)==0,
            % Start new figure if needed
            figure(1); clf;
        end
        subplot(Nrows,Ncols,mod(k-1,Nrows*Ncols)+1);
        plot(TIME_sim,PK_sim,'o-'); hold on
        plot(TIME_obs,PK_obs,'kx','LineWidth',2); hold on
        title(sprintf('ID: %d',allID(k)));
        if mod(k-1,Nrows*Ncols)==Nrows*Ncols-1 || k==length(allID),
            % Export figure if full or last ID plotted
            printFigureSBPOP(gcf,filenameCheckPlot);
        end
    end
end
close all;
convert2pdfSBPOP(filenameCheckPlot)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add a baseline column 
% If BASE column does not exist to get the baseline PD then use the mean of
% the DV assessments before first dose
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allID = unique(dataPKconcFilled.ID);
dataPKconc = dataset();
for k=1:length(allID),
    datak = dataPKconcFilled(dataPKconcFilled.ID==allID(k),:);
    % Handle (non)available BASE column
    try 
        base = datak.BASE;
    catch
        warning('No BASE column in dataset. Use mean of all PD assessments until first dose to calculate baseline.');
        datak.BASE = zeros(length(datak),1);
        datak.BASE(datak.TIM2<=0 & datak.TYPE==TYPE) = 1;
    end
    % Get BASELINE value
    BASELINE = mean(datak.DV(datak.BASE==1));
    % Add baseline to dataset
    datak.PD_BASELINE = BASELINE*ones(length(datak),1);
    % Combine
    dataPKconc = [dataPKconc; datak];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add YTYPE column
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dataPKconc.YTYPE = NaN(length(dataPKconc),1);
dataPKconc.YTYPE(dataPKconc.TYPE==TYPE) = 1;        % PD observation
if length(unique(dataPKconc.YTYPE)) > 1,
    error('Something strange');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define initial structure of popPD dataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varNames = {'STUD' 'SUBJ' 'ID' 'TIM2' 'TAD' 'TIMEUNIT' 'TYPE' 'DV' 'NAME' 'UNIT' 'MDV' 'EVID' 'AMT' 'CENS' 'DOSE' 'TRT' 'REGIMEN' 'YTYPE'};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add covariates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varNames = [varNames covNames(:)' catNames(:)'];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add concentration column
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varNames = [varNames 'PKconcentration']; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add baseline PD column name
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varNames{end+1} = 'PD_BASELINE';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create popPK dataset in defined structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
datapopPD = dataset();
for k=1:length(varNames),
    datapopPD.(varNames{k}) = dataPKconc.(varNames{k});
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate Monolix header
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add PD baseline to the continuous covariates
covNames{end+1} = 'PD_BASELINE';
regressionNames = {'PKconcentration'};
dataheaderMonolix = SBPOPgetMonolixDataHeader(datapopPD,covNames,catNames,regressionNames);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get placebo data only
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
datapopPDplacebo = datapopPD(datapopPD.TRT==0,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate Monolix header for placebo dataset 
% No regression variables and remove cat covs with only one item
% Set AMT and EVID to IGNORE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dataheaderMonolix_PLACEBO = SBPOPgetMonolixDataHeader(datapopPDplacebo,covNames,catNames,{});
dataheaderMonolix_PLACEBO = strrep(dataheaderMonolix_PLACEBO,',EVID,',',IGNORE,');
dataheaderMonolix_PLACEBO = strrep(dataheaderMonolix_PLACEBO,',AMT,',',IGNORE,');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Export datasets
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[p,f,e] = fileparts(analysisDataset);
warning off
mkdir(p);
warning on
% All data
SBPOPexportCSVdataset(datapopPD,[p '/' f '.csv']);
% Placebo only
SBPOPexportCSVdataset(datapopPDplacebo,[p '/' f '_PLACEBO.csv']);
