function [dataheaderMonolix,dataheaderMonolix_PLACEBO] = SBPOPconvert2popPDparametersDataset(model,dosing,path_to_monolix_project_folder,data,TYPE,covNames,catNames,analysisDataset)
% [DESCRIPTION]
% This function converts the given dataset in "data" into a popPD dataset
% where the individual PK parameters are given, to be used as regression
% parameters.
% The datapopPD dataset will contain a subset of the columns in the
% original dataset, sufficient to perform a popPD analysis. 
% Dose records (TYPE=0) are kept to be able to simulate the PK.
% 
% The data need to be provided, following the standard dataspec, defined in
% the help to the function SBPOPcheckDataFormat, so please look there for
% more information.   
%
% Two datasets will be created. One with all subjects and one with only
% Placebo subjects.
%
% For each dose record an additional dose record with AMT=1 and ADM=99 is
% included in the output datasets. INPUT99 can then be used for placebo
% model building.
%
% [SYNTAX]
% [dataheaderMonolix,dataheaderMonolix_PLACEBO] = SBPOPconvert2popPDparametersDataset(model,dosing,path_to_monolix_project_folder,data,TYPE,covNames,catNames,analysisDataset)
%
% [INPUT]
% model:        SBmodel: PKPD model to be used for simulation / estimation
%               - only used to check the parameters in the fit results. 
% dosing:       SBPOPdosing scheme that will be used for simulation.
% path_to_monolix_project_folder: Path to the Monolix PK fit to get the
%                                 individual parameters from 
% data:         MATLAB PKPD dataset in standard data spec format  
% TYPE:         The numeric type identifier (column TYPE), specifying which
%               PD readout is to be modeled and kept in the PD dataset
% covNames:     Cell-array with names of continuous covariates
% catNames:     Cell-array with names of categorical covariates
% analysisDataset:    Filename, including path for saving the popPD dataset
%                     as CSV file. 
%
% [OUTPUT]
% dataheaderMonolix:            Data header for Monolix fits
% dataheaderMonolix_PLACEBO:    Removed "X" and set to IGNORE since the
%                               placebo models only will not contain
%                               regression variables 
%
% [ASSUMPTIONS]
%
% [AUTHOR]
% Henning Schmidt, henning.schmidt@novartis.com
%
% [DATE]
% 9th May 2010
%
% [PLATFORM]
% Windows XP Engine, MODESIM, MATLAB R2009a
%
% [KEYWORDS]
% MATLAB, SBPOP, dataexploration, datacleaning
% 
% [TOOLBOXES USED]
% Statistics Toolbox
%
% [VALIDATION HISTORY]
%
% [MODIFICATION HISTORY]

% Information:
% ============
% Copyright (c) 2012 Novartis Pharma AG
% 
% This program is Free Open Source Software: you can redistribute it and/or modify 
% it under the terms of the GNU General Public License as published by 
% the Free Software Foundation, either version 3 of the License, or 
% (at your option) any later version. 
% 
% This program is distributed in the hope that it will be useful, 
% but WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License 
% along with this program. If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get population parameters (for placebo subjects)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
param                   = SBPOPsampleMONOLIXparam(path_to_monolix_project_folder,0,0);
ParameterNamesFIT       = param.parameterNames;
PopParameterValues      = param.parameterValuesPopulation;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check that the parameters  in the fit result also are in the model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
moddos                  = mergemoddosSBPOP(model,dosing);
paramModdos             = SBparameters(moddos);
IXparametersNotInModel  = [];
for k=1:length(ParameterNamesFIT),
    ix = strmatchSB(ParameterNamesFIT{k},paramModdos,'exact');
    if isempty(ix),
        warning(sprintf('Parameter "%s" available in the fit results but not in the PK model. This parameter will be neglected.\n\tPlease make sure that this does make sense!',ParameterNamesFIT{k}));
        IXparametersNotInModel(end+1) = k;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get individual PK parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
indiv_param             = SBPOPloadNONCSVdataset([path_to_monolix_project_folder '/RESULTS/indiv_parameters.txt']);
indiv_param             = indiv_param(:,1:length(ParameterNamesFIT)+1);
% Remove parameters that are not in the model
indiv_param(:,IXparametersNotInModel+1) = [];
% Remove the _mode thing
indiv_param = set(indiv_param,'VarNames',strrep(get(indiv_param,'VarNames'),'_mode',''));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove parameters from the names and population things if needed (after having gotten the individual PK parameters)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ParameterNamesFIT(IXparametersNotInModel) = [];
PopParameterValues(IXparametersNotInModel) = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove all but dose and PD of interest records from the dataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
data(data.TYPE~=0 & data.TYPE~=TYPE,:) = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create dataset with PK parameters - for placebo subjects use population parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Add population parameters for all
dataPKparam = data;
for k=1:length(ParameterNamesFIT),
    dataPKparam.(ParameterNamesFIT{k}) = PopParameterValues(k)*ones(length(dataPKparam),1);
end

% Add individual parameters for subjects where estimates have been made
allID = unique(dataPKparam.ID);
dataPKparamFilled = dataset();
for k=1:length(allID),
    datak = dataPKparam(dataPKparam.ID==allID(k),:);
    % Check if ID available in indiv_param
    ix = find(indiv_param.ID==allID(k));
    % If available then enter the values
    if ~isempty(ix),
        indiv_param_k = indiv_param(ix,2:end);
        pnames = get(indiv_param_k,'VarNames');
        for k2=1:length(pnames),
            datak.(pnames{k2}) = indiv_param_k.(pnames{k2})(1)*ones(length(datak),1);
        end
    end
    % Combine again
    dataPKparamFilled = [dataPKparamFilled; datak];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add a baseline column 
% If BASE column does not exist to get the baseline PD then use the mean of
% the DV assessments before first dose
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allID = unique(dataPKparamFilled.ID);
dataPKparam = dataset();
for k=1:length(allID),
    datak = dataPKparamFilled(dataPKparamFilled.ID==allID(k),:);
    % Handle (non)available BASE column
    try 
        base = datak.BASE;
    catch
        warning('No BASE column in dataset. Use mean of all PD assessments until first dose to calculate baseline.');
        datak.BASE = zeros(length(datak),1);
        datak.BASE(datak.TIM2<=0 & datak.TYPE==TYPE) = 1;
    end
    % Get BASELINE value
    BASELINE = mean(datak.DV(datak.BASE==1));
    % Add baseline to dataset
    datak.PD_BASELINE = BASELINE*ones(length(datak),1);
    % Combine
    dataPKparam = [dataPKparam; datak];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add YTYPE column
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dataPKparam.YTYPE = NaN(length(dataPKparam),1);
dataPKparam.YTYPE(dataPKparam.TYPE==0) = 0;         % Dose
dataPKparam.YTYPE(dataPKparam.TYPE==TYPE) = 1;      % PD observation
if length(unique(dataPKparam.YTYPE)) > 2,
    error('Something strange');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define initial structure of popPD dataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varNamesData = get(dataPKparam,'VarNames');
if ~isempty(strmatchSB('TINF',varNamesData)),
    % TINF is present
    varNames = {'STUD' 'SUBJ' 'ID' 'TIM2' 'TAD' 'TIMEUNIT' 'TYPE' 'DV' 'NAME' 'UNIT' 'MDV' 'EVID' 'CENS' 'AMT' 'ADM' 'TINF' 'DOSE' 'TRT' 'REGIMEN' 'YTYPE'};
else
    % TINF is not present
    varNames = {'STUD' 'SUBJ' 'ID' 'TIM2' 'TAD' 'TIMEUNIT' 'TYPE' 'DV' 'NAME' 'UNIT' 'MDV' 'EVID' 'CENS' 'AMT' 'ADM' 'DOSE' 'TRT' 'REGIMEN' 'YTYPE'};
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add covariates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varNames = [varNames covNames(:)' catNames(:)'];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add parameter names
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varNames = [varNames ParameterNamesFIT(:)']; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add baseline PD column name
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varNames{end+1} = 'PD_BASELINE';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create popPK dataset in defined structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
datapopPD = dataset();
for k=1:length(varNames),
    datapopPD.(varNames{k}) = dataPKparam.(varNames{k});
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate Monolix header
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add PD baseline to the continuous covariates
covNames{end+1} = 'PD_BASELINE';
regressionNames = ParameterNamesFIT;
dataheaderMonolix = SBPOPgetMonolixDataHeader(datapopPD,covNames,catNames,regressionNames);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% For each dose records a placebo dose record with AMT=0 and ADM=99 is
% added
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
datapopPD_DOSE_PLACEBO      = datapopPD(datapopPD.EVID==1,:);
datapopPD_DOSE_PLACEBO.AMT  = 1*ones(length(datapopPD_DOSE_PLACEBO),1);
datapopPD_DOSE_PLACEBO.ADM  = 99*ones(length(datapopPD_DOSE_PLACEBO),1);
datapopPD                   = [datapopPD; datapopPD_DOSE_PLACEBO];
datapopPD                   = sortrows(datapopPD,{'ID','TIM2','TYPE'});

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get placebo data only
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
datapopPDplacebo = datapopPD(datapopPD.TRT==0,:);
% Remove ADM not 99 and not 0
datapopPDplacebo(datapopPDplacebo.ADM~=0 & datapopPDplacebo.ADM~=99,:) = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate Monolix header for placebo dataset 
% No regression variables and remove cat covs with only one item
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dataheaderMonolix_PLACEBO = SBPOPgetMonolixDataHeader(datapopPDplacebo,covNames,catNames,{});

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Export datasets
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[p,f,e] = fileparts(analysisDataset);
warning off
mkdir(p);
warning on
% All data
SBPOPexportCSVdataset(datapopPD,[p '/' f '.csv']);
% Placebo only
SBPOPexportCSVdataset(datapopPDplacebo,[p '/' f '_PLACEBO.csv']);
