function [dataheaderMonolix] = SBPOPconvert2popPKPDsimultaneousDataset(data,TYPE,covNames,catNames,analysisDataset)
% [DESCRIPTION]
% This function converts the given dataset in "data" into a popPKPD dataset
% for simultaneous PK and PD parameter fit.
%
% Assumption: In the model OUTPUT1 is PK and OUTPUT2 is PD
% 
% The data need to be provided, following the standard dataspec, defined in
% the help to the function SBPOPcheckDataFormat, so please look there for
% more information.   
%
% Only one dataset will be generated, including both placebo and treated
% subjects.
%
% For each dose record an additional dose record with AMT=1 and ADM=99 is
% included in the output dataset. INPUT99 can then be used for placebo
% model building.
%
% [SYNTAX]
% [dataheaderMonolix] = SBPOPconvert2popPKPDsimultaneousDataset(data,TYPE,covNames,catNames,analysisDataset)
%
% [INPUT]
% data:         MATLAB PKPD dataset in standard data spec format  
% TYPE:         The numeric type identifier (column TYPE), specifying which
%               PD readout is to be modeled and kept in the PD dataset
% covNames:     Cell-array with names of continuous covariates
% catNames:     Cell-array with names of categorical covariates
% analysisDataset:    Filename, including path for saving the popPD dataset
%                     as CSV file. 
%
% [OUTPUT]
% dataheaderMonolix:    Data header for Monolix
%
% [ASSUMPTIONS]
%
% [AUTHOR]
% Henning Schmidt, henning.schmidt@novartis.com
%
% [DATE]
% 9th May 2010
%
% [PLATFORM]
% Windows XP Engine, MODESIM, MATLAB R2009a
%
% [KEYWORDS]
% MATLAB, SBPOP, dataexploration, datacleaning
% 
% [TOOLBOXES USED]
% Statistics Toolbox
%
% [VALIDATION HISTORY]
%
% [MODIFICATION HISTORY]

% Information:
% ============
% Copyright (c) 2012 Novartis Pharma AG
% 
% This program is Free Open Source Software: you can redistribute it and/or modify 
% it under the terms of the GNU General Public License as published by 
% the Free Software Foundation, either version 3 of the License, or 
% (at your option) any later version. 
% 
% This program is distributed in the hope that it will be useful, 
% but WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License 
% along with this program. If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove all but dose, PK, and PD of interest records from the dataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
data(data.TYPE~=0 & data.TYPE~=1 & data.TYPE~=TYPE,:) = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add a baseline column 
% If BASE column does not exist to get the baseline PD then use the mean of
% the DV assessments before first dose
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allID = unique(data.ID);
dataPKPD = dataset();
for k=1:length(allID),
    datak = data(data.ID==allID(k),:);
    % Handle (non)available BASE column
    try 
        base = datak.BASE;
    catch
        warning('No BASE column in dataset. Use mean of all PD assessments until first dose to calculate baseline.');
        datak.BASE = zeros(length(datak),1);
        datak.BASE(datak.TIM2<=0 & datak.TYPE==TYPE) = 1;
    end
    % Get BASELINE value
    BASELINE = mean(datak.DV(datak.BASE==1));
    % Add baseline to dataset
    datak.PD_BASELINE = BASELINE*ones(length(datak),1);
    % Combine
    dataPKPD = [dataPKPD; datak];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add YTYPE column
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dataPKPD.YTYPE = NaN(length(dataPKPD),1);
dataPKPD.YTYPE(dataPKPD.TYPE==0)      = 0;      % Dose
dataPKPD.YTYPE(dataPKPD.TYPE==1)      = 1;      % PK observation
dataPKPD.YTYPE(dataPKPD.TYPE==TYPE)   = 2;      % PD observation
if length(unique(dataPKPD.YTYPE)) > 3,
    error('Something strange');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define initial structure of popPD dataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varNamesData = get(dataPKPD,'VarNames');
if ~isempty(strmatchSB('TINF',varNamesData)),
    % TINF is present
    varNames = {'STUD' 'SUBJ' 'ID' 'TIM2' 'TAD' 'TIMEUNIT' 'TYPE' 'DV' 'NAME' 'UNIT' 'MDV' 'EVID' 'CENS' 'AMT' 'ADM' 'TINF' 'DOSE' 'TRT' 'REGIMEN' 'YTYPE'};
else
    % TINF is not present
    varNames = {'STUD' 'SUBJ' 'ID' 'TIM2' 'TAD' 'TIMEUNIT' 'TYPE' 'DV' 'NAME' 'UNIT' 'MDV' 'EVID' 'CENS' 'AMT' 'ADM' 'DOSE' 'TRT' 'REGIMEN' 'YTYPE'};
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add covariates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varNames = [varNames covNames(:)' catNames(:)'];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add baseline PD column name
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varNames{end+1} = 'PD_BASELINE';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create popPK dataset in defined structure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
datapopPKPD = dataset();
for k=1:length(varNames),
    datapopPKPD.(varNames{k}) = dataPKPD.(varNames{k});
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate Monolix header
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add PD baseline to the continuous covariates
covNames{end+1}     = 'PD_BASELINE';
regressionNames     = {};
dataheaderMonolix   = SBPOPgetMonolixDataHeader(datapopPKPD,covNames,catNames,regressionNames);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% For each dose records a placebo dose record with AMT=0 and ADM=99 is
% added
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
datapopPD_DOSE_PLACEBO      = datapopPKPD(datapopPKPD.EVID==1,:);
datapopPD_DOSE_PLACEBO.AMT  = 1*ones(length(datapopPD_DOSE_PLACEBO),1);
datapopPD_DOSE_PLACEBO.ADM  = 99*ones(length(datapopPD_DOSE_PLACEBO),1);
datapopPKPD                 = [datapopPKPD; datapopPD_DOSE_PLACEBO];
datapopPKPD                 = sortrows(datapopPKPD,{'ID','TIM2','TYPE'});

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Export dataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[p,f,e] = fileparts(analysisDataset);
warning off
mkdir(p);
warning on
% All data
SBPOPexportCSVdataset(datapopPKPD,[p '/' f '.csv']);
