function [MODEL_INFO] = SBPOPbuildPopPKModelSpace(nameSpace, modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix, optionsModelSpace)
% [DESCRIPTION]
% This functions allows to assess a user defined popPK model subspace.
% Models within this subspace will be created / run / results will be
% imported / fitness assessments will be generated / table to compare
% different models will also be generated.
% 
% Since this function is quite comprehensive, it is impossible to handle
% ALL possible PK model structures. The goal is to handle the most common
% ones. Applying the Pareto principle.
%
% Currently covered are:
%  1,2,3 compartmental IV, Bolus, 1st order absorption models
%  IV, Bolus and absorption can happen in the same dataset
%  Linear and linear+saturable clearance
%  Lagtime on 1st order absorption
%
% Not covered:
%   IOV
%   Nonlinear bioavailability
%   Different error models for same output and subgrouping
%
% [SYNTAX]
% [MODEL_INFO] = SBPOPbuildPopPKModelSpace(nameSpace, modeltest, analysisDatasetFile, dataheaderMonolix)
% [MODEL_INFO] = SBPOPbuildPopPKModelSpace(nameSpace, modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix)
% [MODEL_INFO] = SBPOPbuildPopPKModelSpace(nameSpace, modeltest, analysisDatasetFile, dataheaderMonolix, optionsMonolix, optionsModelSpace)
%
% [INPUT]
% nameSpace:                    Unique identifier for giving the output folders and files a name
% modeltest:                    Matlab structure with model space information
%
%    DEFINITIONS REQUIRED FOR:
%       modeltest.numberCompartments        = [1 2 3];                          Define number of compartments to test
%       modeltest.errorModels               = {'const','comb1','prop','exp'};   Define residual error models to test
%       modeltest.saturableClearance        = [0 1];                            Test both linear and linear+saturable clearance
%       modeltest.lagTime                   = [0 1];                            Test both no lag time and lagtime
%       modeltest.FACTOR_UNITS              = 1;                                Conversion factor for dose and concentration units
%       modeltest.POPvalues0                = [ 0.5   0.2   2    10    10    10   10   0.5    5    0.2    0.5];
%                                             This can also be a cell-array with multiple vectors for initial guesses
%       modeltest.POPestimate               = [  0     1    1     1     1     1   1     1     1     0       0];
%                                             This can also be a cell-array with multiple vectors for estimation definitions
%       modeltest.IIVestimate               = [  0     1    1     1     1     1   1     1     1     0       0];
%                                             This can also be a cell-array with multiple vectors for estimation definitions
%  			                                    0: IIV not estimated (IIVvalues0 not used) 
%           			                        1: IIV estimated (IIVvalues0 as starting guesses)
%                       		                2: IIV not estimated but fixed on IIVvalues0 value
%
%    DEFINITIONS OPTIONAL FOR:
%       modeltest.IIVvalues0                = [  1     1    1     1     1     1   1     1     1     1       1];
%                                             This should always be only a vector - no need for testing different initial guesses here
%                                             It is optional. By default all IIVs start on 1
%       modeltest.covarianceModels          = Can be a cell-array - all definitions will be combined with all others
%
%       modeltest.covariateModels           = Can be a cell-array - all definitions will be combined with all others
%       modeltest.covariateModelValues      = Can be a cell-array - will not be combined with all others but will be matched with covariateModels
%                                             If not defined all covariate coefficient estimations start from 0
%       modeltest.COVestimate               = Can be a cell-array - will not be combined with all others but will be matched with covariateModels
%                                             If not defined all covariate coefficient are estimated
%
% analysisDatasetFile:          Relative path including filename to the popPK dataset from where this function is called 
%
% dataHeaderMonolix:            Data set header definition for Monolix
%
% optionsMonolix:               Options for Monolix algorithm settings
%       optionsMonolix.LLsetting:              'linearization', 'importantSampling', or 'both'. Default: 'both'
%       optionsMonolix.algorithm.SEED:         Seed setting. Defualt: 123456
%       optionsMonolix.algorithm.K1:           First iterations. Default: 500
%       optionsMonolix.algorithm.K2:           Final iterations. Default: 200
%       optionsMonolix.algorithm.K1_AUTO:      Automatic first iteration number (0: off, 1: on). Default: 0
%       optionsMonolix.algorithm.K2_AUTO:      Automatic final iteration number (0: off, 1: on). Default: 0
%       optionsMonolix.algorithm.NRCHAINS:     Number of parallel chains. Default: 1
%       optionsMonolix.PathMonolixStandalone:  Path to the Monolix standalone file (Monolix.sh or Monolix.bat) (including filename)
%
% optionsModelSpace:            Structure with additional optional information
%       optionsModelSpace.buildModelsOnly:      =0 (default): build models and run them. =1: build models only, but to not run them
%       optionsModelSpace.Ntests:               Number of tests to perform (default=1)
%       optionsModelSpace.createGOFplots:       Only used if Ntests>1. =0: don't, =1: do (default: 0)
%       optionsModelSpace.std_noise_setting:    Standard deviation to use to add exponential noise to the initial parameter guesses
%                                               Parameter_perturbed = Parameter_guess*exp(std_noise_setting*randn)
%                                               (default=0.5)
%
% [OUTPUT]
% - MODEL_INFO structure with all results about the models
% - All PK models are created in the ['../Models/' nameSpace 'model'] folder 
% - A modelInfo.txt file is written into the ['../Models/' nameSpace 'model'] folder, detailing what the different models contain
% - Fit analysis plots are produced and stored in the ['../Output/FitAnalysis/' nameSpace 'model/Plots'] folder
% - Text results for parameters are produced in the ['../Output/FitAnalysis/' nameSpace 'model/Info'] folder 
%
% - The function also save the MODEL_INFO as .mat file in ['../Output/FitAnalysis/' nameSpace 'model/Info/MODEL_INFO.mat']
%
% [AUTHOR]
% Henning Schmidt, henning.schmidt@novartis.com
%
% [DATE]
% 2nd March, 2013

% Information:
% ============
% Copyright (c) 2012 Novartis Pharma AG
% 
% This program is Free Open Source Software: you can redistribute it and/or modify 
% it under the terms of the GNU General Public License as published by 
% the Free Software Foundation, either version 3 of the License, or 
% (at your option) any later version. 
% 
% This program is distributed in the hope that it will be useful, 
% but WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License 
% along with this program. If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Handle undefined path to standalone Monolix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
try PathMonolixStandalone = optionsMonolix.PathMonolixStandalone;   catch, PathMonolixStandalone        = '';   end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Handle repeat fit settings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
try buildModelsOnly     = optionsModelSpace.buildModelsOnly;        catch, buildModelsOnly      = 0;            end
try Ntests              = optionsModelSpace.Ntests;                 catch, Ntests               = 1;            end
try std_noise_setting   = optionsModelSpace.std_noise_setting;      catch, std_noise_setting    = 0.5;          end
try createGOFplots      = optionsModelSpace.createGOFplots;         catch, createGOFplots       = 0;            end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define output location and project name settings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
modelProjectsFolder             = ['../Models/' nameSpace 'model'];
dataRelPathFromProjectPath      = '../../../Data';
PROJECT_PREFIX                  = ['FIT' nameSpace '_'];
FitanalysisPlotsFolder          = ['../Output/FitAnalysis/' nameSpace 'model/Plots'];
FitInfoOutputFolder             = ['../Output/FitAnalysis/' nameSpace 'model/Info'];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Determine covNames and catNames based on monolixHeader and analysisdataset
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
headerDataSet = SBPOPloadCSVdataset(analysisDatasetFile,1);
covNames = headerDataSet(strmatchSB('COV',explodePCSB(dataheaderMonolix),'exact'));
catNames = headerDataSet(strmatchSB('CAT',explodePCSB(dataheaderMonolix),'exact'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sample Ntests popvalues
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if Ntests > 1,
    % Repeated fits from different initial guesses desired
    popvalues0  = modeltest.POPvalues0;
    popestimate = modeltest.POPestimate;
    iivestimate = modeltest.IIVestimate;
    % Check if single set of initial guesses given, otherwise error
    if iscell(popvalues0),
        if length(popvalues0) > 1,
            error('The repeated fit setting in "optionsModelSpace" can only be used if "modeltest.POPvalues0" given as a single vector.');
        else
            popvalues0 = popvalues0{1};
        end
    end
    if ~isvector(popvalues0),
        error('The repeated fit setting in "optionsModelSpace" can only be used if "modeltest.POPvalues0" given as a single vector.');
    end
    if ~isvector(popestimate),
        error('The repeated fit setting in "optionsModelSpace" can only be used if "modeltest.POPestimate" given as a single vector, not as a cell-array!');
    end
    if ~isvector(iivestimate),
        error('The repeated fit setting in "optionsModelSpace" can only be used if "modeltest.IIVestimate" given as a single vector, not as a cell-array!');
    end
    
    % Get info about parameters where perturbation allowed (both fixed and
    % random effects need to be estimated)
    randomize_parameters = popestimate.*iivestimate;
    % Sample initial guesses
    popvaluesnew = {};
    for k=1:Ntests,
        popvaluesnew{k} = popvalues0.*exp(std_noise_setting*randomize_parameters.*randn(1,length(popvalues0)));
    end
    modeltest.POPvalues0 = popvaluesnew;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create the popPK model projects 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[MODEL_INFO] = buildPKmodelSpaceSBPOP(modelProjectsFolder, dataRelPathFromProjectPath, PROJECT_PREFIX,...
                    analysisDatasetFile, dataheaderMonolix, modeltest, optionsMonolix);

if ~buildModelsOnly,

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Check if a matlabpool is available (then parallel is possible)
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    parallelFlag = 0;
    try
        nnodes = matlabpool('Size');
        if nnodes>1,
            parallelFlag = 1;
        end
    catch
        parallelFlag = 0;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Check availability of standalone monolix, then allow parallel runs
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if parallelFlag==1 && isempty(PathMonolixStandalone),
        parallelFlag = 0;
        warning('You could run things in parallel, but path to Monolix standalone version required for that.');
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Run the popPK model projects
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Get the model projects
    projects = dir([modelProjectsFolder '/' PROJECT_PREFIX '*']);
    % Run the model projects - use parfor to allow for parallel evaluation of the models if parallel toolbox available
    warning off
    oldpath = pwd();
    if parallelFlag,
        % parallel uses always standalone Monolix version 
        parfor k=1:length(projects),
            fprintf('Running project %d of %d ...\n',k,length(projects));
            cd([modelProjectsFolder '/' projects(k).name]);
            try
                system([PathMonolixStandalone ' -p ./project.mlxtran -nowin -f run']);
            catch
                warning on;
                warning('Estimation of model "%d" had issues. Estimation aborted.',k);
                warning off;
            end
            cd(oldpath);
        end
    else
        % Non parallel runs with either Matlab or Standalone version of
        % Monolix
        for k=1:length(projects),
            try
                fprintf('Running project %d of %d ...\n',k,length(projects));
                if isempty(PathMonolixStandalone),
                    SBPOPrunMONOLIXproject([modelProjectsFolder '/' projects(k).name]);
                else
                    cd([modelProjectsFolder '/' projects(k).name]);
                    system([PathMonolixStandalone ' -p ./project.mlxtran -nowin -f run']);
                    cd(oldpath);
                end
            catch
                warning on;
                warning('Estimation of model "%d" had issues. Estimation aborted.',k);
                warning off;
                cd(oldpath);
            end
        end        
    end
    warning on;        
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Read the results
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Get the model projects
    projects = dir([modelProjectsFolder '/' PROJECT_PREFIX '*']);
    % Read run results
    RESULTS = [];
    for k=1:length(projects),
        try
            x = parseMONOLIXresultsSBPOP([modelProjectsFolder '/' projects(k).name]);
            y = sampleMONOLIXpopulationParametersSBPOP(x,0,1);
            
            % Get objective function values
            if strcmp(optionsMonolix.LLsetting,'linearization'),
                LL = x.objectivefunction.LL_lin;
                AIC = x.objectivefunction.AIC_lin;
                BIC = x.objectivefunction.BIC_lin;
            else
                LL = x.objectivefunction.LL_is;
                AIC = x.objectivefunction.AIC_is;
                BIC = x.objectivefunction.BIC_is;
            end
            
            % Collect results
            RESULTS(k).model                            = projects(k).name;
            RESULTS(k).LL                               = LL;
            RESULTS(k).AIC                              = AIC;
            RESULTS(k).BIC                              = BIC;
            RESULTS(k).parameternames                   = x.parameters.names;
            RESULTS(k).parametervalues                  = x.parameters.values;
            RESULTS(k).stderrors                        = x.parameters.stderrors;
            RESULTS(k).correlationmatrixRandomEffects   = y.randomEffects.correlationmatrix;
            RESULTS(k).rawParameterInfo                 = x.rawParameterInfo;
        catch
            % It might happen that some model was not run ...
            % Collect results
            RESULTS(k).model                            = projects(k).name;
            RESULTS(k).LL                               = NaN;
            RESULTS(k).AIC                              = NaN;
            RESULTS(k).BIC                              = NaN;
            RESULTS(k).parameternames                   = {};
            RESULTS(k).parametervalues                  = [];
            RESULTS(k).stderrors                        = NaN;
            RESULTS(k).correlationmatrixRandomEffects   = [];
            RESULTS(k).rawParameterInfo                 = [];
        end
    end
    % Determine rankink after BIC
    ranking_var = sortrows([[1:length(projects)]' [RESULTS.BIC]'],2); %#ok<*NBRAK>
    RANKING = ranking_var(:,1);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Update the MODEL_INFO.MODEL_SETTINGS.POPvalues0 field with the estimated fixed effect parameters
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    for k=1:length(MODEL_INFO),
        MODEL_INFO(k).POPvalues0_analytic  = MODEL_INFO(k).POPvalues0;
        MODEL_INFO(k).POPestimate_analytic = MODEL_INFO(k).POPestimate;
        MODEL_INFO(k).IIVestimate_analytic = MODEL_INFO(k).IIVestimate;
        
        MODEL_INFO(k).POPvalues0  = NaN(1,11);
        MODEL_INFO(k).POPestimate = zeros(1,11);
        MODEL_INFO(k).IIVestimate = zeros(1,11);
        
        MODEL_INFO(k).POPvalues0(MODEL_INFO(k).ANALYTIC_ODE_TRANSFORMATION.INDEXODE)  = MODEL_INFO(k).POPvalues0_analytic;
        MODEL_INFO(k).POPestimate(MODEL_INFO(k).ANALYTIC_ODE_TRANSFORMATION.INDEXODE)  = MODEL_INFO(k).POPestimate_analytic;
        MODEL_INFO(k).IIVestimate(MODEL_INFO(k).ANALYTIC_ODE_TRANSFORMATION.INDEXODE)  = MODEL_INFO(k).IIVestimate_analytic;
        
        MODEL_INFO(k).RESULTS                   = RESULTS(k);
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Print the results for the fits
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    printResultsModelSpaceSBPOP(FitInfoOutputFolder, MODEL_INFO, RANKING, PROJECT_PREFIX)
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Safe MODEL_INFO
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    save([FitInfoOutputFolder '/MODEL_INFO.mat'],'MODEL_INFO');
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Do the fit analysis plots etc.
    % Do that only if Ntests==1 If Ntests>1 then it still can be done if createGOFplots~=0
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if Ntests == 1 || createGOFplots,
        % Get the model projects
        projects = dir([modelProjectsFolder '/' PROJECT_PREFIX '*']);
        % Load the data
        datacontents = SBPOPloadCSVdataset(analysisDatasetFile);
        % Clean the output folder
        warning off; %#ok<*WNOFF>
        try rmdir(FitanalysisPlotsFolder,'s'); catch, end; mkdir(FitanalysisPlotsFolder);
        warning on; %#ok<*WNON>
        % Do the fit analysis plots - use parfor to allow for parallel generation
        % of the results if parallel toolbox available
        if parallelFlag,
            % Parallel
            parfor k=1:length(projects),
                try
                    projectPath = [modelProjectsFolder '/' projects(k).name];
                    optionsFA = [];
                    optionsFA.basefilename = [FitanalysisPlotsFolder '/' projects(k).name '_'];
                    SBPOPfitanalysisPopPK(datacontents,projectPath,covNames,catNames,optionsFA)
                catch
                end
            end
        else
            % Non-parallel
            for k=1:length(projects),
                try
                    projectPath = [modelProjectsFolder '/' projects(k).name];
                    optionsFA = [];
                    optionsFA.basefilename = [FitanalysisPlotsFolder '/' projects(k).name '_'];
                    SBPOPfitanalysisPopPK(datacontents,projectPath,covNames,catNames,optionsFA)
                catch
                end
            end
        end
    end
end
