function [] = SBPOPfitanalysisETAvsCOV(data,projectPath,covNames,catNames,options)
% [DESCRIPTION]
% This function is used to plot the individual variations over covariates
% and categorical covariates. It can be used for a first assessment which
% variables could be potentially interesting covariates in the model.
%
% For the analysis of the categorical covariates information from the
% modeling dataset is used. This means that if transformations of
% categorical covariates have been done in the model, the generated
% information is of no big use (this is the case only for categorical
% covariates).
%
% [SYNTAX]
% [] = SBPOPfitanalysisETAvsCOV(data,projectPath,covNames,catNames)
% [] = SBPOPfitanalysisETAvsCOV(data,projectPath,covNames,catNames,options)
%
% [INPUT]
% data:         Analysis dataset used for the Monolix Fit (needed to get
%               information about categorical covariates)
% projectPath:  Path to a Monolix project folder. The results of the
%               Monolix run need to be stored in a "RESULTS" folder in this
%               path. The file "indiv_eta.txt" needs to be present in
%               this RESULTS folder.
% covNames:     Cell-array with names of continuous covariates
% catNames:     Cell-array with names of categorical covariates
% options:      MATLAB structure with plotting optins:
%                   
%                   options.corrcoeffThreshold: number between 0 and 1. If
%                          correlation above this value, then data plotted in red.
%                          (default: 0.3)
%                   options.filename:   If a filename is provided, then the results are exported
%                                       into a postscript (windows) or PDf (unix) document with this name.
%
% [OUTPUT]
% Plots, ETAs over covariates
%
% [ASSUMPTIONS]
%
% [AUTHOR]
% Henning Schmidt, henning.schmidt@novartis.com
%
% [DATE]
% 15th April 2010
%
% [PLATFORM]
% Windows XP Engine, MATLAB R2009a, MATLAB
%
% [KEYWORDS]
% MATLAB, SBPOP, covariate search, covariate
% 
% [TOOLBOXES USED]
% Statistics Toolbox
%
% [VALIDATION HISTORY]
%
% [MODIFICATION HISTORY]
 
% Information:
% ============
% Copyright (c) 2012 Novartis Pharma AG
% 
% This program is Free Open Source Software: you can redistribute it and/or modify 
% it under the terms of the GNU General Public License as published by 
% the Free Software Foundation, either version 3 of the License, or 
% (at your option) any later version. 
% 
% This program is distributed in the hope that it will be useful, 
% but WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License 
% along with this program. If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Handle varargins
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
try corrcoeffThreshold = options.corrcoeffThreshold; catch, corrcoeffThreshold = 0.3; end
try filename = options.filename; catch, filename = ''; end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check cov and catnames
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
datanames = get(data,'VarNames');
for k=1:length(covNames),
    if isempty(strmatchSB(covNames{k},datanames,'exact')), error('The dataset does not contain the covariate ''%s''.',covNames{k}); end    
end
for k=1:length(catNames),
    if isempty(strmatchSB(catNames{k},datanames,'exact')), error('The dataset does not contain the covariate ''%s''.',catNames{k}); end    
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Warn the user about potential issues with categorical data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!');
disp('SBPOPfitanalysisETAvsCOV: For the analysis of the categorical covariates information from the');
disp('modeling dataset is used. This means that if transformations of');
disp('categorical covariates have been done in the model, the generated');
disp('information might not be of use (this is the case only for categorical');
disp('covariates).');
disp('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Construct RESULTS path
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
resultsPath = [projectPath '/RESULTS'];
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check the projectPath
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~exist(resultsPath),
    error(sprintf('The provided project path "%s" does not point to a valid SBPOP/Monolix project.\nPlease make sure a "RESULTS" folder is in the provided path.',projectPath));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check that indiv_eta.txt is present in the RESULTS folder
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
indiv_eta_file = [resultsPath '/indiv_eta.txt'];
if ~exist(indiv_eta_file)
    error('The "indiv_eta.txt" file does not exist in the RESULTS folder.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load eta file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
indiv_eta   = SBPOPloadNONCSVdataset([resultsPath '/indiv_eta.txt']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Determine random effect estimates for shrinkage determination
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x = parseMONOLIXresultsSBPOP(projectPath);
y = sampleMONOLIXpopulationParametersSBPOP(x,0,1);
OMEGA       = y.randomEffects.values;
OMEGAnames  = y.randomEffects.names;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get eta modes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dataeta = dataset();
dataeta.ID = indiv_eta.ID;
for k=1:length(OMEGAnames),
    dataeta.(OMEGAnames{k}) = indiv_eta.(['eta_' OMEGAnames{k} '_mode']);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove the non estimated omegas/etas
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ix =  find(sum(abs(double(dataeta(:,2:end)))) ~= 0);
dataeta_est                 = dataeta(:,[1 ix+1]);
OMEGAnames_est              = OMEGAnames(ix);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the continuous covariates - transformed or not from indiv_eta
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
datacovs = dataset();
datacovs.ID = indiv_eta.ID;
varnames = get(indiv_eta,'VarNames');
for k=1:length(covNames),
    ix = strmatchSB(covNames{k},varnames);
    covname = covNames{k};
    if isempty(ix),
        ix = strmatchSB(['t_' covNames{k}],varnames);
        covname = ['t_' covNames{k}];
    end
    if isempty(ix),
        error('Trouble finding the right covariate - check!');
    end
    datacovs.(covname) = indiv_eta.(covname);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the categorical covariates for same IDs as in the dataeta_est
% This only works correclty if no transformation has been done in Monolix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
allIDeta = unique(dataeta_est.ID);
datacats = dataset();
for k=1:length(allIDeta),
    datak = data(data.ID==allIDeta(k),:);
    datacatsk = dataset();
    datacatsk.ID = allIDeta(k);
    for k2=1:length(catNames),
        datacatsk.(catNames{k2}) = datak.(catNames{k2})(1);
    end
    datacats = [datacats; datacatsk];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Interface to old code ;-)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
etas = dataeta_est(:,2:end);
covs = datacovs(:,2:end);
cats = datacats(:,2:end);
nretas = size(etas,1);
nrcovs = size(covs,1);
nrcats = size(cats,1);
etaNames = get(etas,'VarNames');
covNames = get(covs,'VarNames');
catNames = get(cats,'VarNames');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Determine subplot organization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Neta = length(etaNames);
nrow = ceil(sqrt(Neta));
ncol = ceil(Neta/nrow);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% If filename then remove old file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(filename),
    startNewPrintFigureSBPOP(filename);
    % also create path if not yet created
    [p,f,e] = fileparts(filename);
    warning off
    mkdir(p);
    warning on
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First handle continuous covariates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Cycle through covariates and produce on figure per covariate
% The etas in subplots
for k=1:size(covs,2),
    cov = double(covs(:,k));
    % New figure
    h = figure;
    set(h,'Name',['Covariate: ' covNames{k}])
    for k2=1:size(etas,2),
        name = etaNames{k2};
        eta = double(etas(:,k2));
        subplot(nrow,ncol,k2);
        cc = corrcoef([cov,eta]);
        cc = cc(1,2);
        if abs(cc) > corrcoeffThreshold,            
            plot(cov,eta,'or'); hold on
        else
            plot(cov,eta,'o'); hold on
        end
        % Add linear regression result
        X = [ones(size(cov)) cov];
        warning off
        b = regress(eta,X); % Removes NaN data
        warning on
        x = get(gca,'XLim');        
        plot(x, b(1)+b(2)*x,'k--')
        % Title etc.
        title(['Corr. coeff.: ' sprintf('%1.2g',cc)],'Interpreter','None');
        xlabel(covNames{k},'Interpreter','None')
        ylabel(etaNames{k2},'Interpreter','None')
    end
    set(h,'Color',[1 1 1]);
    if ~isempty(filename),
        printFigureSBPOP(gcf,filename);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Second handle categorical covariates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Cycle through covariates and produce on figure per covariate
% The etas in subplots
for k=1:size(cats,2),
    cat = double(cats(:,k));
    catunique = unique(cat);
    % New figure
    h = figure;
    set(h,'Name',['Covariate: ' catNames{k}]);
    for k2=1:size(etas,2),
        name = etaNames{k2};
        eta = double(etas(:,k2));
        subplot(nrow,ncol,k2);
        boxplot(eta,cat);
        plotZeroLim = get(gca,'XLim');
        hold on;
        plot(plotZeroLim,[0 0],'--k')
        xlabel(catNames{k},'Interpreter','None')
        ylabel(etaNames{k2},'Interpreter','None')
    end
    set(h,'Color',[1 1 1]);    
    if ~isempty(filename),
        printFigureSBPOP(gcf,filename);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PS2PDF
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(filename),
    convert2pdfSBPOP(filename);
    close all;
end
