function [legendText,outputALL] = simulateVPCSBPOP(projectFolder,model,output,obsTimes,dosings,covNames,catNames,dataCOV,dataCAT,regressionVariables,dataREGRESSION,options)
% [DESCRIPTION]
% This function is an auxiliary function for VPC generation.
%
% [SYNTAX]
% [legendText] = simulateVPCSBPOP(projectFolder,model,output,dosing,obsTimes,covNames,catNames,dataCOV,dataCAT,options)
%
% [INPUT]
% projectFolder:    Cell-array with the name of the Monolix project folder.
%                   Needs to include the full path to this folder
% model:            Structural model fitting to the Monolix fit to use for
%                   the simulation  
% output:           String with the name of the model variable to compare
% obsTimes:         Observation times to use for plotting
% dosings:          Cell-array with dosing schemes matching in order with
%                   the dataCOV and dataCAT entries. For sampling each
%                   corresponding dosing scheme, dataCOV and dataCAT will
%                   be selected
% covNames:         Cell-array with names of continuous covariates. Only 
%                   the ones used in the model will be considered 
% catNames:         Cell-array with names of categorical covariates. Only 
%                   the ones used in the model will be considered 
% dataCOV:          MATLAB dataset with continuous covariates in same
%                   order as in covNames
%                   From these, the Nsim covariate combinations will be
%                   sampled using a uniform distribution
% dataCAT:          MATLAB dataset with categorical covariates in same
%                   order as in covNames
%                   From these, the Nsim covariate combinations will be
%                   sampled using a uniform distribution
% regressionVariables: Cell-array with names of regression parameters that
%                   are defined in the dataset and which need to be passed
%                   to the model. These parameters will also be sampled.
% dataREGRESSION:   MATLAB dataset with regression variable values in same
%                   order as in regressionVariables
%                   From these, the Nsim regression variable combinations
%                   will be sampled using a uniform distribution
%
% options:          Matlab structure with optional information
%       options.Nsim                Number of samples from IIV
%                                   distributions (default: 0, only
%                                   population mean will be compared). If
%                                   Nsim=1 it will be set to Nsim=2 - that
%                                   small values anyway dont really make
%                                   sense but Nsim=1 would be messy and
%                                   lead to an error
%       options.quantiles           Vector with quantiles to compute for
%                                   comparison (does only make sense if
%                                   Nsim reasonably large) (default: [0.05 0.95])
%       options.logY                =1: log Y axis, =0: linear Y axis
%       options.optionsIntegrator   options for the integration.
%                                   By default: abstol=1e-6, reltol=1e-6
%
% [OUTPUT]
%
% [AUTHOR]
% Henning Schmidt, henning.schmidt@novartis.com
%
% [DATE]
% 4th April, 2013

% Information:
% ============
% Copyright (c) 2012 Novartis Pharma AG
% 
% This program is Free Open Source Software: you can redistribute it and/or modify 
% it under the terms of the GNU General Public License as published by 
% the Free Software Foundation, either version 3 of the License, or 
% (at your option) any later version. 
% 
% This program is distributed in the hope that it will be useful, 
% but WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
% GNU General Public License for more details. 
% 
% You should have received a copy of the GNU General Public License 
% along with this program. If not, see <http://www.gnu.org/licenses/>.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Handle options
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
try Nsim                = options.Nsim;                  catch, Nsim                 = 0;                end
try quantiles           = options.quantiles;             catch, quantiles            = [0.05 0.95];      end
try logY                = options.logY;                  catch, logY                 = 1;                end
try optionsIntegrator   = options.optionsIntegrator;     catch, optionsIntegrator    = [];               end

try optionsIntegrator.abstol = optionsIntegrator.abstol; catch, optionsIntegrator.abstol = 1e-6;         end
try optionsIntegrator.reltol = optionsIntegrator.reltol; catch, optionsIntegrator.reltol = 1e-6;         end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adjust Nsim if needed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if Nsim==1,
    Nsim = 2;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Handle logY
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if logY==1,
    plotType = 'semilogy';
else
    plotType = 'plot';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get colors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[colors,lines]  = getcolorsSBPOP();

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Sample Nsim covariate and regression variable values from the provided information
% Use uniform distribution for sampling
% Also sample the corresponding dosings - needed to be sure to match
% relative dosing (mg/kg, mg/m2, etc.)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
NdataSubjects = length(dataCOV);
sampleCovariateDosingIX = ceil(NdataSubjects*rand(1,Nsim));
if ~isempty(dataCOV),
    COVvaluesSampled = double(dataCOV(sampleCovariateDosingIX,:));
else
    COVvaluesSampled = [];
    covNames = {};
end
if ~isempty(dataCAT),
    CATvaluesSampled = double(dataCAT(sampleCovariateDosingIX,:));
else
    CATvaluesSampled = [];
    catNames = {};
end
if ~isempty(dataREGRESSION),
    REGRESSIONvaluesSampled = double(dataREGRESSION(sampleCovariateDosingIX,:));
else
    REGRESSIONvaluesSampled = [];
    regressionVariables = {};
end
DosingsSampled   = dosings(sampleCovariateDosingIX);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Merge model and create MEX model
% Same general dosing scheme always - use first for merging
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
moddos          = mergemoddosSBPOP(model,DosingsSampled{1});
SBPDmakeMEXmodel(moddos,'mexModel');
dummy = feval('mexModel');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Get individual parameters for model simulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
parameters = SBPOPsampleMONOLIXparam(projectFolder,0,Nsim,  covNames, COVvaluesSampled, catNames, CATvaluesSampled);
paramNames  = parameters.parameterNames;
paramValuesInd = parameters.parameterValuesIndividual;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check that parameters in all fits are available in the model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
modelparamnames = SBparameters(moddos);
for k2=1:length(paramNames),
    ix = strmatchSB(paramNames{k2}, modelparamnames, 'exact');
    if isempty(ix),
        error('simulateVPCSBPOP: Parameters provided in the fit results ("projectFolder") need to be present in the structural model ("model").');
    end
end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulate all models for all individual 
% samples with individual dosing schemes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(sprintf('Simulating individual parameters ...'));
% Get space for simulation results
outputALL  = NaN(length(obsTimes),Nsim);
if ~isempty(regressionVariables),
    paramChangeNames  = [paramNames regressionVariables];
    paramChangeValues = [paramValuesInd REGRESSIONvaluesSampled]; 
else
    paramChangeNames = paramNames;    
    paramChangeValues = [paramValuesInd]; 
end
parfor k2=1:Nsim,
    % Simulate the model
    try
        simres = SBPOPsimdosing('mexModel',DosingsSampled{k2},obsTimes,[],paramChangeNames,paramChangeValues(k2,:),optionsIntegrator);
        % Get output
        outputALL(:,k2) = simres.variablevalues(:,variableindexSB(moddos,output));
    catch
        outputALL(:,k2) = NaN(length(obsTimes),1);
        disp(lasterr);
    end
end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove mexModel
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear mex
delete(['mexModel.' mexext]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get the statistics
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
quantileInfoALL_sampled = quantile(outputALL',quantiles);
medianInfoALL_sampled   = quantile(outputALL',0.5);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot the results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(1); clf;
legendText = {};
% Plot population mean
feval(plotType,obsTimes,medianInfoALL_sampled,'-','Color',colors(1,:),'LineWidth',3); hold on
% Create legend text
legendText{end+1} = sprintf('Simulation - Median');
% Plot sampled results
for k2=1:length(quantiles),
    % Plot quantiles for sampling
    feval(plotType,obsTimes,quantileInfoALL_sampled(k2,:),'-','Color',colors(1,:),'LineWidth',2); hold on
    % create legend text
    legendText{end+1} = sprintf('Simulation - Quantile: %g',quantiles(k2));
end
legend(legendText,'Location','best','Interpreter','none');
set(gca,'FontSize',12);
grid on;
xlabel('Time','FontSize',14);
ylabel(output,'FontSize',14);
