function y = isSBML_Reaction(varargin)
% isSBML_Reaction(SBMLStructure, Level, Version(optional)) 
% checks that SBMLStructure represents a reaction 
% within an sbml model of the specified level
% 
% if SBMLStructure represents a reaction within an SBML model
% it has the appropriate fields 
% eg    Typecode
%       Metaid (L2V1)
%       Notes
%       Annotations
%       SBOTerm (L2V2)
%       Name
%       Id (L2V1)
%       ListReactants
%       ListProducts
%       ListModifiers (L2V1)
%       KineticLaw
%       Reversible
%       Fast
%       isSetFast (L2V1)
%
% NOTE: content of brackets indicates the level and version of sbml from which the given field
% is appropriate.
%
% Returns 1 if SBMLStructure is a structure containing each of the above
% fields (appropriate with the given level and version) 
% and the typecode is "SBML_REACTION"
% 
% Returns 0 if SBMLStructure is not a structure 
% or does not contain one of the appropriate fields
% or the typecode is not "SBML_REACTION"

%  Filename    :   isSBML_Reaction.m
%  Description :
%  Author(s)   :   SBML Development Group <sbml-team@caltech.edu>
%  $Id: isSBML_Reaction.m,v 1.10 2008/01/18 21:16:53 sarahkeating Exp $
%  $Source v $
%
%<!---------------------------------------------------------------------------
% This file is part of SBMLToolbox.  Please visit http://sbml.org for more
% information about SBML, and the latest version of SBMLToolbox.
%
% Copyright 2005-2007 California Institute of Technology.
% Copyright 2002-2005 California Institute of Technology and
%                     Japan Science and Technology Corporation.
% 
% This library is free software; you can redistribute it and/or modify it
% under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation.  A copy of the license agreement is provided
% in the file named "LICENSE.txt" included with this software distribution.
% and also available online as http://sbml.org/software/sbmltoolbox/license.html
%----------------------------------------------------------------------- -->

if (nargin < 2 || nargin > 3)
    error('wrong number of input arguments');
end;

SBMLStructure = varargin{1};
Level = varargin{2};

if (nargin == 3)
    Version = varargin{3};
else
    Version = 1;
end;

if (Level == 1)
    SBMLfieldnames = {'typecode', 'notes', 'annotation','name', 'reactant', 'product', 'kineticLaw', ...
        'reversible', 'fast'};
    nNumberFields = 9;
else
    if (Version == 1)
        SBMLfieldnames = {'typecode', 'metaid', 'notes', 'annotation','name', 'id', 'reactant', 'product', ...
            'modifier', 'kineticLaw', 'reversible', 'fast', 'isSetFast'};
        nNumberFields = 13;
    elseif (Version == 2)
        SBMLfieldnames = {'typecode', 'metaid', 'notes', 'annotation','name', 'id', 'reactant', 'product', ...
            'modifier', 'kineticLaw', 'reversible', 'fast', 'sboTerm', 'isSetFast'};
        nNumberFields = 14;
    elseif (Version == 3)
        SBMLfieldnames = {'typecode', 'metaid', 'notes', 'annotation','sboTerm', 'name', 'id', 'reactant', 'product', ...
            'modifier', 'kineticLaw', 'reversible', 'fast', 'isSetFast'};
        nNumberFields = 14;
    end;
end;
    
typecode = 'SBML_REACTION';

bSBML = 0;

% check that Model is a structure
bSBML = isstruct(SBMLStructure);

% check it contains each of the fields listed
index = 1;
while (bSBML == 1 && index <= nNumberFields)
    bSBML = isfield(SBMLStructure, SBMLfieldnames(index));
    index = index + 1;
end;

% catch cases where isSetFast field was capitalised (Version 1.0.2)
if (Level == 2 && bSBML == 0)
    % may have failed on case
    if (index == 13)
        bSBML = isfield(SBMLStructure, 'IsSetFast');
    end;
end;
    
    
% check that it contains only the fields listed
if (bSBML == 1)
    names = fieldnames(SBMLStructure);
    [m,n] = size(names);
    if (m ~= nNumberFields)
        bSBML = 0;
    end;
end;

% check that the typecode is correct
if (bSBML == 1)
    type = SBMLStructure.typecode;
    k = strcmp(type, typecode);
    if (k ~= 1)
        bSBML = 0;
    end;
end;
    
% check that any nested structures are appropriate
if(bSBML == 1)
    index = 1;
    [x, nNumberReactants] = size(SBMLStructure.reactant);
    while (bSBML == 1 && index <= nNumberReactants)
        bSBML = isSBML_SpeciesReference(SBMLStructure.reactant(index), Level, Version);
        index = index + 1;
    end;

    index = 1;
    [x, nNumberProducts] = size(SBMLStructure.product);
    while (bSBML == 1 && index <= nNumberProducts)
        bSBML = isSBML_SpeciesReference(SBMLStructure.product(index), Level, Version);
        index = index + 1;
    end;

    if (Level == 2)
        index = 1;
        [x, nNumberModifiers] = size(SBMLStructure.modifier);
        while (bSBML == 1 && index <= nNumberModifiers)
            bSBML = isSBML_ModifierSpeciesReference(SBMLStructure.modifier(index), Level, Version);
            index = index + 1;
        end;
    end;

    % if a kinetic law is present check that it is valid
    if (bSBML == 1 && ~isempty(SBMLStructure.kineticLaw))
        bSBML = isSBML_KineticLaw(SBMLStructure.kineticLaw, Level, Version);
    end;
end;
y = bSBML;
